# -*- coding: utf-8 -*-

import copy
import os
import re
import shutil
import tempfile
import unittest

from mock import MagicMock, Mock, patch
from pathlib import Path

import tala.utils
from tala.model.ddd import DDD
from tala.model.grammar.grammar import GrammarBase
from tala.model.grammar.intent import Request, Question, Answer
from tala.model.grammar.required_entity import RequiredPropositionalEntity, RequiredSortalEntity
from tala.nl.languages import ENGLISH
from tala.model.domain import Domain
from tala.model.goal import ResolveGoal
from tala.model.lambda_abstraction import LambdaAbstractedPredicateProposition
from tala.model.ontology import Ontology
from tala.model.predicate import Predicate
from tala.model.question import WhQuestion
from tala.model.sort import Sort, CustomSort
from tala.nl.rasa.generating import generator
from tala.nl.rasa.generating.generator import RasaGenerator
from tala.nl.rasa.generating.examples import Examples, EnglishExamples, SortNotSupportedException


class GeneratorTestsBase(object):
    def setup(self):
        self._temp_dir = tempfile.mkdtemp(prefix="GeneratorTests")
        self._cwd = os.getcwd()
        os.chdir(self._temp_dir)
        self._mocked_ddd = self._create_mocked_ddd()
        self._generator = None
        self._mocked_grammar = None
        self._expected_data = None
        self._grammar_reader_patcher = self._create_grammar_reader_patcher()
        self._result = None

    def tearDown(self):
        os.chdir(self._cwd)
        shutil.rmtree(self._temp_dir)
        self._grammar_reader_patcher.stop()

    def _create_mocked_ddd(self):
        ontology = Mock(spec=Ontology)
        ontology.get_individuals_of_sort.return_value = list()
        ontology.get_ddd_specific_actions.return_value = set()
        ontology.get_sorts.return_value = dict()
        ontology.get_predicates.return_value = dict()
        ontology.get_predicate.side_effect = Exception()
        domain = Mock(spec=Domain)
        domain.get_all_resolve_goals.return_value = []
        ddd = Mock(spec=DDD)
        ddd.ontology = ontology
        ddd.domain = domain
        ddd.grammars = {}
        return ddd

    def _create_grammar_reader_patcher(self):
        patcher = patch("%s.GrammarReader" % generator.__name__, autospec=True)
        MockGrammarReader = patcher.start()
        MockGrammarReader.xml_grammar_exists_for_language.return_value = True
        return patcher

    def given_ddd_name(self, name):
        self._mocked_ddd.name = name

    def given_ontology(
        self,
        sort,
        individuals,
        predicate,
        is_builtin=False,
        is_integer_sort=False,
        is_string_sort=False,
        is_real_sort=False,
        is_datetime_sort=False
    ):
        mocked_sort = Mock(spec=Sort)
        mocked_sort.get_name.return_value = sort
        mocked_sort.is_builtin.return_value = is_builtin
        mocked_sort.is_integer_sort.return_value = is_integer_sort
        mocked_sort.is_string_sort.return_value = is_string_sort
        mocked_sort.is_real_sort.return_value = is_real_sort
        mocked_sort.is_domain_sort.return_value = False
        mocked_sort.is_datetime_sort.return_value = is_datetime_sort
        self._mocked_ddd.ontology.individual_sort.return_value = mocked_sort
        self._mocked_ddd.ontology.get_individuals_of_sort.return_value = individuals
        self._mocked_ddd.ontology.get_sorts.return_value = {sort: mocked_sort}
        self._mocked_ddd.ontology.get_sort.return_value = mocked_sort
        mocked_predicate = Mock(spec=Predicate)
        mocked_predicate.get_name.return_value = predicate
        mocked_predicate.getSort.return_value = mocked_sort
        self._mocked_ddd.ontology.get_predicates.return_value = {predicate: mocked_predicate}
        self._mocked_ddd.ontology.get_predicate.side_effect = None
        self._mocked_ddd.ontology.get_predicate.return_value = mocked_predicate

    def given_actions_in_ontology(self, actions):
        self._mocked_ddd.ontology.get_ddd_specific_actions.return_value = actions

    def given_mocked_grammar(self, requests=None, questions=None, individuals=None, answers=None, strings=None):
        self._mocked_grammar = Mock(spec=GrammarBase)
        self._mocked_grammar.requests_of_action.return_value = requests or []
        self._mocked_grammar.questions_of_predicate.return_value = questions or []
        self._mocked_grammar.answers.return_value = answers or []
        self._mocked_grammar.strings_of_predicate.return_value = strings or []
        individuals = individuals or {}

        def get_individual(name):
            return individuals[name]

        self._mocked_grammar.entries_of_individual.side_effect = get_individual
        self._mocked_ddd.grammars["eng"] = self._mocked_grammar

    def given_changed_to_temp_folder(self):
        temp_dir = tempfile.mkdtemp(prefix="RasaGeneratorTestCase")
        return tala.float_comparison.chdir(temp_dir)

    def given_generator(self):
        self._generator = RasaGenerator(self._mocked_ddd, ENGLISH)

    def when_generate(self):
        self._result = self._generate()

    def _generate(self):
        return self._generator.generate()

    def when_generating_data_with_mocked_grammar_then_exception_is_raised_matching(
        self, expected_exception, expected_pattern
    ):
        try:
            self._generate()
            assert False, "%s not raised" % expected_exception
        except expected_exception as e:
            assert re.match(expected_pattern, str(e))

    def given_expected_plan_questions_in_domain(self, predicates):
        def resolve_goals_of_questions(questions):
            for question in questions:
                mocked_goal = Mock(spec=ResolveGoal)
                mocked_goal.get_question.return_value = question
                yield mocked_goal

        plan_questions = list(self._plan_questions(predicates))
        mocked_resolve_goals = list(resolve_goals_of_questions(plan_questions))
        self._mocked_ddd.domain.get_all_resolve_goals.return_value = mocked_resolve_goals

    def _plan_questions(self, predicates):
        for predicate_name, sort_name in predicates.iteritems():
            predicate = Predicate("mocked_ontology", predicate_name, CustomSort("mocked_ontology", sort_name))
            proposition = LambdaAbstractedPredicateProposition(predicate, "mocked_ontology")
            question = WhQuestion(proposition)
            yield question

    def _action_example(self, action, text, entities=None):
        identifier = "rasa_test:action::%s" % action
        return self._intent_example(identifier, text, entities)

    def _question_example(self, predicate, text, entities=None):
        identifier = "rasa_test:question::%s" % predicate
        return self._intent_example(identifier, text, entities)

    def _intent_example(self, identifier, text, entities=None):
        entities = entities or []
        return {
            "text": text,
            "intent": identifier,
            "entities": entities,
        }

    def _action_examples_with_sortal_entities(self, action, texts, sort, entity_names):
        identifier = "sort:%s" % sort
        return self._action_examples_with_entities(action, texts, identifier, entity_names)

    def _question_examples_with_sortal_entities(self, predicate_in_question, texts, sort, entity_names):
        identifier = "sort:%s" % sort
        return self._question_examples_with_entities(predicate_in_question, texts, identifier, entity_names)

    def _action_examples_with_entities(self, action, texts, entity_identifier, entity_names):
        intent_identifier = "rasa_test:action::%s" % action
        return self._intent_examples_with_entities(intent_identifier, texts, entity_identifier, entity_names)

    def _question_examples_with_entities(self, predicate, texts, entity_identifier, entity_names):
        intent_identifier = "rasa_test:question::%s" % predicate
        return self._intent_examples_with_entities(intent_identifier, texts, entity_identifier, entity_names)

    def _answer_examples_with_sortal_entities(self, sort, entity_names):
        intent_identifier = "rasa_test:answer"
        entity_identifier = "sort:%s" % sort
        return self._intent_examples_with_entities(intent_identifier, [""], entity_identifier, entity_names)

    def _answer_examples_with_propositional_entities(self, predicate, texts, entity_names):
        intent_identifier = "rasa_test:answer"
        entity_identifier = "predicate:%s" % predicate
        return self._intent_examples_with_entities(intent_identifier, texts, entity_identifier, entity_names)

    def _answer_negation_examples_with_sortal_entities(self, sort, entity_names):
        intent_identifier = "rasa_test:answer_negation"
        entity_identifier = "sort:%s" % sort
        return self._intent_examples_with_entities(intent_identifier, ["not "], entity_identifier, entity_names)

    def _answer_negation_examples_with_propositional_entities(self, predicate, texts, entity_names):
        intent_identifier = "rasa_test:answer_negation"
        entity_identifier = "predicate:%s" % predicate
        return self._intent_examples_with_entities(intent_identifier, texts, entity_identifier, entity_names)

    def _intent_examples_with_entities(self, intent_identifier, original_texts, entity_identifier, entity_names):
        start = len(original_texts[0])
        for entity_name in entity_names:
            entities = list(self._entity_examples(entity_identifier, [entity_name], start))
            texts = copy.deepcopy(original_texts)
            texts.insert(1, entity_name)
            text = "".join(texts)
            yield self._intent_example(intent_identifier, text, entities)

    def _entity_examples(self, entity_identifier, entity_names, start):
        for name in entity_names:
            yield {"start": start, "end": start + len(name), "value": name, "entity": entity_identifier}

    def _action_examples_with_propositional_entities(self, action, texts, predicate, entity_names):
        identifier = "predicate:%s" % predicate
        return self._action_examples_with_entities(action, texts, identifier, entity_names)

    def _question_examples_with_propositional_entities(self, question_predicate, texts, answer_predicate, entity_names):
        entity_identifier = "predicate:%s" % answer_predicate
        return self._question_examples_with_entities(question_predicate, texts, entity_identifier, entity_names)

    def _assert_common_examples_in_generated_data_is(self, expected_common_examples):
        actual_common_examples = self._result["rasa_nlu_data"]["common_examples"]
        for expected_common_example in expected_common_examples:
            assert expected_common_example in actual_common_examples, "Expected to find %s in %s but didn't" % (
                expected_common_example, actual_common_examples
            )

    def then_common_examples_have_been_generated(self, expected_common_examples):
        self._assert_common_examples_in_generated_data_is(expected_common_examples)

    def then_no_common_examples_have_been_generated_for_answer_negation(self):
        actual_common_examples = self._result["rasa_nlu_data"]["common_examples"]
        actual_negation_examples = [
            example for example in actual_common_examples if example["intent"] == "rasa_test:answer_negation"
        ]
        assert len(actual_negation_examples) == 0, "Expected no answer negations but got %s" % actual_negation_examples


class GenerateAndWriteTests(GeneratorTestsBase, unittest.TestCase):
    def setUp(self):
        GeneratorTestsBase.setup(self)
        self._MockUTF8FileWriter = None
        self._mocked_file_writer = None

    @patch("{}.UTF8FileWriter".format(generator.__name__), autospec=True)
    def test_written_data_when_calling_generate_and_write_to_file(self, MockUTF8FileWriter):
        self.given_mocked_file_writer(MockUTF8FileWriter)
        self.given_generator()
        self.given_generate_returns({"data": "mock"})
        self.when_calling_generate_and_write_to_file()
        self.then_written_data_was('{\n    "data": "mock"\n}')

    def given_mocked_file_writer(self, MockUTF8FileWriter):
        self._MockUTF8FileWriter = MockUTF8FileWriter
        self._mocked_file_writer = MockUTF8FileWriter.return_value

    def given_generate_returns(self, data):
        self._generator.generate = Mock()
        self._generator.generate.return_value = data

    def when_calling_generate_and_write_to_file(self):
        self._generator.generate_and_write_to_file()

    def then_written_data_was(self, expected_data):
        self._mocked_file_writer.write.assert_called_once_with(expected_data)

    @patch("{}.UTF8FileWriter".format(generator.__name__), autospec=True)
    def test_path_when_calling_generate_and_write_to_file(self, MockUTF8FileWriter):
        self.given_mocked_file_writer(MockUTF8FileWriter)
        self.given_generator()
        self.given_generate_returns({"data": "mock"})
        self.when_calling_generate_and_write_to_file()
        self.then_path_was(Path("build_rasa") / "eng" / "rasa_data.json")

    def then_path_was(self, expected_path):
        self._MockUTF8FileWriter.assert_called_once_with(expected_path)

    @patch("{}.UTF8FileWriter".format(generator.__name__), autospec=True)
    def test_directories_created_when_calling_generate_and_write_to_file(self, MockUTF8FileWriter):
        self.given_mocked_file_writer(MockUTF8FileWriter)
        self.given_generator()
        self.given_generate_returns({"data": "mock"})
        self.when_calling_generate_and_write_to_file()
        self.then_directories_was_created()

    def then_directories_was_created(self):
        self._mocked_file_writer.create_directories.assert_called_once_with()


class UnsupportedBuiltinSortGeneratorTestCase(GeneratorTestsBase, unittest.TestCase):
    def setUp(self):
        GeneratorTestsBase.setup(self)

    def test_generate_requests(self):
        self.given_ddd_name("mocked_ddd")
        self.given_ontology(sort="real", predicate="selected_price", individuals=[], is_builtin=True, is_real_sort=True)
        self.given_actions_in_ontology({"purchase"})
        self.given_mocked_grammar(requests=self._requests_of_action("purchase", "selected_price"))
        self.given_generator()
        self.when_generating_data_with_mocked_grammar_then_exception_is_raised_matching(
            SortNotSupportedException, "Builtin sort 'real' is not yet supported together with RASA"
        )

    def _requests_of_action(self, action, required_predicate):
        return [
            Request(action, ["take a note that ", ""], [
                RequiredPropositionalEntity(required_predicate),
            ]),
        ]

    def test_generate_questions(self):
        self.given_ddd_name("mocked_ddd")
        self.given_ontology(sort="real", predicate="selected_price", individuals=[], is_builtin=True, is_real_sort=True)
        self.given_expected_plan_questions_in_domain({"selected_price": "real"})
        self.given_mocked_grammar(questions=self._questions_of_predicate("selected_price", "real"))
        self.given_generator()
        self.when_generating_data_with_mocked_grammar_then_exception_is_raised_matching(
            SortNotSupportedException, "Builtin sort 'real' is not yet supported together with RASA"
        )

    def _questions_of_predicate(self, question_predicate, sort):
        return [
            Question(
                question_predicate, ["how long time remains of the ", " reminder"], [
                    RequiredSortalEntity(sort),
                ]
            ),
        ]

    def test_generate_answers(self):
        self.given_ddd_name("mocked_ddd")
        self.given_ontology(sort="real", predicate="selected_price", individuals=[], is_builtin=True, is_real_sort=True)
        self.given_mocked_grammar()
        self.given_generator()
        self.when_generating_data_with_mocked_grammar_then_exception_is_raised_matching(
            SortNotSupportedException, "Builtin sort 'real' is not yet supported together with RASA"
        )


class CustomSortGeneratorTestCase(GeneratorTestsBase, unittest.TestCase):
    def setUp(self):
        GeneratorTestsBase.setup(self)

    def given_ontology_with_individuals(
        self, sort, predicate, is_integer_sort=False, is_string_sort=False, is_real_sort=False
    ):
        individuals = [
            "contact_john",
            "contact_john_chi",
            "contact_lisa",
            "contact_mary",
            "contact_andy",
            "contact_andy_chi",
        ]
        super(CustomSortGeneratorTestCase,
              self).given_ontology(sort, individuals, predicate, is_integer_sort, is_string_sort, is_real_sort)

    def given_mocked_grammar_with_individuals(self, requests=None, questions=None, answers=None):
        individuals = {
            "contact_john": ["John", "Johnny"],
            "contact_john_chi": [u"约翰"],
            "contact_lisa": ["Lisa", "Elizabeth"],
            "contact_mary": ["Mary"],
            "contact_andy": ["Andy"],
            "contact_andy_chi": [u"安迪"],
        }
        super(CustomSortGeneratorTestCase, self).given_mocked_grammar(requests, questions, individuals, answers)

    def test_generate_requests(self):
        self.given_ddd_name("rasa_test")
        self.given_ontology_with_individuals(sort="contact", predicate="selected_contact_to_call")
        self.given_actions_in_ontology({"call"})
        self.given_mocked_grammar_with_individuals(requests=self._requests_of_action)
        self.given_generator()
        self.when_generate()
        self.then_common_examples_have_been_generated_for_action("call", "contact", "selected_contact_to_call")

    @property
    def _requests_of_action(self):
        return [
            Request("call", ["make a call"], []),
            Request("call", ["call ", ""], [
                RequiredSortalEntity("contact"),
            ]),
            Request("call", ["make a call to ", ""], [
                RequiredPropositionalEntity("selected_contact_to_call"),
            ]),
        ]

    def then_common_examples_have_been_generated_for_action(self, action, sort, predicate_of_propositional_answer):
        def examples():
            yield self._action_example(action, "make a call")
            for example in self._action_examples_with_sortal_entities(action, ["call "], sort, self._contact_data):
                yield example
            for example in self._action_examples_with_propositional_entities(
                action, ["make a call to "], predicate_of_propositional_answer, self._contact_data
            ):
                yield example

        self._assert_common_examples_in_generated_data_is(examples())

    def test_generate_requests_with_two_answers(self):
        self.given_ddd_name("rasa_test")
        self.given_ontology_with_individuals(sort="contact", predicate="caller")
        self.given_actions_in_ontology({"call"})
        self.given_mocked_grammar_with_individuals(
            requests=[
                Request(
                    "call", ["call ", " and say hi from ", ""], [
                        RequiredSortalEntity("contact"),
                        RequiredPropositionalEntity("caller"),
                    ]
                ),
            ]
        )
        self.given_generator()
        self.when_generate()
        self.then_common_examples_have_been_generated([
            {
                "text":
                "call John and say hi from Lisa",
                "intent":
                "rasa_test:action::call",
                "entities": [{
                    "start": 5,
                    "end": 9,
                    "value": "John",
                    "entity": "sort:contact"
                }, {
                    "start": 26,
                    "end": 30,
                    "value": "Lisa",
                    "entity": "predicate:caller"
                }]
            },
            {
                "text":
                "call Johnny and say hi from Lisa",
                "intent":
                "rasa_test:action::call",
                "entities": [{
                    "start": 5,
                    "end": 11,
                    "value": "Johnny",
                    "entity": "sort:contact"
                }, {
                    "start": 28,
                    "end": 32,
                    "value": "Lisa",
                    "entity": "predicate:caller"
                }]
            },
            {
                "text":
                "call John and say hi from Elizabeth",
                "intent":
                "rasa_test:action::call",
                "entities": [{
                    "start": 5,
                    "end": 9,
                    "value": "John",
                    "entity": "sort:contact"
                }, {
                    "start": 26,
                    "end": 35,
                    "value": "Elizabeth",
                    "entity": "predicate:caller"
                }]
            },
            {
                "text":
                "call Johnny and say hi from Elizabeth",
                "intent":
                "rasa_test:action::call",
                "entities": [{
                    "start": 5,
                    "end": 11,
                    "value": "Johnny",
                    "entity": "sort:contact"
                }, {
                    "start": 28,
                    "end": 37,
                    "value": "Elizabeth",
                    "entity": "predicate:caller"
                }]
            },
            {
                "text":
                "call Lisa and say hi from John",
                "intent":
                "rasa_test:action::call",
                "entities": [{
                    "start": 5,
                    "end": 9,
                    "value": "Lisa",
                    "entity": "sort:contact"
                }, {
                    "start": 26,
                    "end": 30,
                    "value": "John",
                    "entity": "predicate:caller"
                }]
            },
            {
                "text":
                "call Lisa and say hi from Johnny",
                "intent":
                "rasa_test:action::call",
                "entities": [{
                    "start": 5,
                    "end": 9,
                    "value": "Lisa",
                    "entity": "sort:contact"
                }, {
                    "start": 26,
                    "end": 32,
                    "value": "Johnny",
                    "entity": "predicate:caller"
                }]
            },
            {
                "text":
                "call Elizabeth and say hi from John",
                "intent":
                "rasa_test:action::call",
                "entities": [{
                    "start": 5,
                    "end": 14,
                    "value": "Elizabeth",
                    "entity": "sort:contact"
                }, {
                    "start": 31,
                    "end": 35,
                    "value": "John",
                    "entity": "predicate:caller"
                }]
            },
            {
                "text":
                "call Elizabeth and say hi from Johnny",
                "intent":
                "rasa_test:action::call",
                "entities": [{
                    "start": 5,
                    "end": 14,
                    "value": "Elizabeth",
                    "entity": "sort:contact"
                }, {
                    "start": 31,
                    "end": 37,
                    "value": "Johnny",
                    "entity": "predicate:caller"
                }]
            },
        ])

    def test_generate_questions(self):
        self.given_ddd_name("rasa_test")
        self.given_ontology_with_individuals(sort="contact", predicate="selected_contact_to_call")
        self.given_expected_plan_questions_in_domain({"phone_number_of_contact": "contact"})
        self.given_mocked_grammar_with_individuals(questions=self._questions_of_predicate)
        self.given_generator()
        self.when_generate()
        self.then_common_examples_have_been_generated_for_question(
            "phone_number_of_contact", "contact", "selected_contact_of_phone_number"
        )

    @property
    def _questions_of_predicate(self):
        return [
            Question("phone_number_of_contact", ["tell me a phone number"], []),
            Question("phone_number_of_contact", ["what is ", "'s number"], [
                RequiredSortalEntity("contact"),
            ]),
            Question(
                "phone_number_of_contact", ["tell me ", "'s number"], [
                    RequiredPropositionalEntity("selected_contact_of_phone_number"),
                ]
            ),
        ]

    def then_common_examples_have_been_generated_for_question(
        self, predicate_in_question, sort, predicate_of_propositional_answer
    ):
        def examples():
            yield self._question_example(predicate_in_question, "tell me a phone number")
            for example in self._question_examples_with_sortal_entities(
                predicate_in_question, ["what is ", "'s number"], sort, self._contact_data
            ):
                yield example
            for example in self._question_examples_with_propositional_entities(
                predicate_in_question, ["tell me ", "'s number"], predicate_of_propositional_answer, self._contact_data
            ):
                yield example

        self._assert_common_examples_in_generated_data_is(examples())

    def then_common_examples_have_been_generated_for_questions_and_exected_data(self):
        self.then_common_examples_have_been_generated_for_questions(self._expected_data)

    def test_generate_answer_intents(self):
        self.given_ddd_name("rasa_test")
        self.given_ontology_with_individuals(sort="contact", predicate="selected_contact_to_call")
        self.given_mocked_grammar_with_individuals()
        self.given_generator()
        self.when_generate()
        self.then_common_examples_have_been_generated_for_answer("contact")

    def then_common_examples_have_been_generated_for_answer(self, sort):
        def examples():
            for example in self._answer_examples_with_sortal_entities(sort, self._contact_data):
                yield example

        self._assert_common_examples_in_generated_data_is(examples())

    def test_generate_answer_negation_intents(self):
        self.given_ddd_name("rasa_test")
        self.given_ontology_with_individuals(sort="contact", predicate="selected_contact_to_call")
        self.given_mocked_grammar_with_individuals()
        self.given_generator()
        self.when_generate()
        self.then_common_examples_have_been_generated_for_answer_negation("contact")

    def then_common_examples_have_been_generated_for_answer_negation(self, sort):
        def examples():
            for example in self._answer_negation_examples_with_sortal_entities(sort, self._contact_data):
                yield example

        self._assert_common_examples_in_generated_data_is(examples())

    @property
    def _contact_data(self):
        return ["Andy", "Mary", "Lisa", "Elizabeth", u"安迪", u"约翰", "John", "Johnny"]

    def test_answers(self):
        self.given_ddd_name("rasa_test")
        self.given_ontology_with_individuals(sort="contact", predicate="selected_contact")
        self.given_mocked_grammar_with_individuals(answers=self._answers("selected_contact"))
        self.given_generator()
        self.when_generate()
        self.then_common_examples_have_been_generated_for_propositional_answer("selected_contact")

    def _answers(self, answer_predicate):
        return [
            Answer(["my friend ", ""], [
                RequiredPropositionalEntity(answer_predicate),
            ]),
        ]

    def then_common_examples_have_been_generated_for_propositional_answer(self, predicate):
        def examples():
            for example in self._answer_examples_with_propositional_entities(
                predicate, ["my friend ", ""], self._contact_data
            ):
                yield example

        self._assert_common_examples_in_generated_data_is(examples())


class BuiltinSortGeneratorTestCase(GeneratorTestsBase, unittest.TestCase):
    def setUp(self):
        GeneratorTestsBase.setup(self)
        self._mock_builtin_sort_examples = ["mock example 1", "mock example 2"]
        self._examples_patcher = self._create_examples_patcher()

    def tearDown(self):
        GeneratorTestsBase.tearDown(self)
        self._examples_patcher.stop()

    def _create_examples_patcher(self):
        def create_mock_examples():
            mock_examples = MagicMock(spec=Examples)
            mock_examples.get_builtin_sort_examples.return_value = self._mock_builtin_sort_examples
            return mock_examples

        patcher = patch("%s.Examples" % generator.__name__)
        mock_examples_patcher = patcher.start()
        mock_examples_patcher.from_language.return_value = create_mock_examples()
        return patcher

    def test_generate_requests(self):
        self.given_ddd_name("rasa_test")
        self.given_ontology(sort="mock_sort", individuals=[], predicate="mock_predicate", is_builtin=True)
        self.given_actions_in_ontology({"mock_action"})
        self.given_mocked_grammar(
            requests=[
                Request("mock_action", ["mock phrase without entities"], []),
                Request("mock_action", ["mock phrase with sortal entity ", ""], [RequiredSortalEntity("mock_sort")]),
                Request(
                    "mock_action", ["mock phrase with propositional entity ", ""],
                    [RequiredPropositionalEntity("mock_predicate")]
                )
            ]
        )
        self.given_generator()
        self.when_generate()
        self.then_common_examples_have_been_generated(
            [self._action_example("mock_action", "mock phrase without entities")] + list(
                self._action_examples_with_sortal_entities(
                    "mock_action", ["mock phrase with sortal entity "], "mock_sort", self._mock_builtin_sort_examples
                )
            ) + list(
                self._action_examples_with_propositional_entities(
                    "mock_action", ["mock phrase with propositional entity "], "mock_predicate", self.
                    _mock_builtin_sort_examples
                )
            )
        )

    def test_generate_questions(self):
        self.given_ddd_name("rasa_test")
        self.given_ontology(sort="mock_sort", individuals=[], predicate="mock_predicate", is_builtin=True)
        self.given_expected_plan_questions_in_domain(predicates={"mock_predicate": "mock_sort"})
        self.given_mocked_grammar(
            questions=[
                Question("mock_predicate", ["mock phrase without entities"], []),
                Question(
                    "mock_predicate", ["mock phrase with sortal entity ", ""], [RequiredSortalEntity("mock_sort")]
                ),
                Question(
                    "mock_predicate", ["mock phrase with propositional entity ", ""],
                    [RequiredPropositionalEntity("mock_predicate")]
                )
            ]
        )
        self.given_generator()
        self.when_generate()
        self.then_common_examples_have_been_generated(
            [self._question_example("mock_predicate", "mock phrase without entities")] + list(
                self._question_examples_with_sortal_entities(
                    "mock_predicate", ["mock phrase with sortal entity "], "mock_sort", self._mock_builtin_sort_examples
                )
            ) + list(
                self._question_examples_with_propositional_entities(
                    "mock_predicate", ["mock phrase with propositional entity "], "mock_predicate", self.
                    _mock_builtin_sort_examples
                )
            )
        )

    def test_generate_answer_intents(self):
        self.given_ddd_name("rasa_test")
        self.given_ontology(sort="mock_sort", individuals=[], predicate="mock_predicate", is_builtin=True)
        self.given_mocked_grammar()
        self.given_generator()
        self.when_generate()
        self.then_common_examples_have_been_generated(
            self._answer_examples_with_sortal_entities("mock_sort", self._mock_builtin_sort_examples)
        )

    def test_generate_answer_negation_intents(self):
        self.given_ddd_name("rasa_test")
        self.given_ontology(sort="mock_sort", individuals=[], predicate="mock_predicate", is_builtin=True)
        self.given_mocked_grammar()
        self.given_generator()
        self.when_generate()
        self.then_common_examples_have_been_generated(
            self._answer_negation_examples_with_sortal_entities("mock_sort", self._mock_builtin_sort_examples)
        )

    def test_answers(self):
        self.given_ddd_name("rasa_test")
        self.given_ontology(sort="mock_sort", individuals=[], predicate="mock_predicate", is_builtin=True)
        self.given_mocked_grammar(
            answers=[
                Answer(["mock phrase with propositional entity ", ""], [RequiredPropositionalEntity("mock_predicate")])
            ]
        )
        self.given_generator()
        self.when_generate()
        self.then_common_examples_have_been_generated(
            self._answer_examples_with_propositional_entities(
                "mock_predicate", ["mock phrase with propositional entity ", ""], self._mock_builtin_sort_examples
            )
        )


class StringSortGeneratorTestCase(GeneratorTestsBase, unittest.TestCase):
    def setUp(self):
        GeneratorTestsBase.setup(self)

    def test_examples_extended_with_strings_of_predicate(self):
        self.given_ddd_name("rasa_test")
        self.given_ontology(
            sort="string", predicate="mock_predicate", individuals=[], is_builtin=True, is_string_sort=True
        )
        self.given_actions_in_ontology({"mock_action"})
        self.given_mocked_grammar(
            requests=[
                Request("mock_action", ["mock phrase without entities"], []),
                Request("mock_action", ["mock phrase with sortal entity ", ""], [RequiredSortalEntity("string")]),
                Request(
                    "mock_action", ["mock phrase with propositional entity ", ""],
                    [RequiredPropositionalEntity("mock_predicate")]
                )
            ],
            strings=["mock string of predicate"]
        )
        self.given_generator()
        self.when_generate()
        self.then_common_examples_have_been_generated(
            [self._action_example("mock_action", "mock phrase without entities")] + list(
                self._action_examples_with_string_entities(
                    "mock_action", ["mock phrase with sortal entity "],
                    EnglishExamples().string
                )
            ) + list(
                self._action_examples_with_propositional_entities(
                    "mock_action", ["mock phrase with propositional entity "], "mock_predicate",
                    EnglishExamples().string + ["mock string of predicate"]
                )
            )
        )

    def _action_examples_with_string_entities(self, action, texts, entity_names):
        return self._action_examples_with_entities(action, texts, "string", entity_names)

    def test_do_not_generate_answer_negation_intents(self):
        self.given_ddd_name("rasa_test")
        self.given_ontology(
            sort="string", predicate="selected_message", individuals=[], is_builtin=True, is_string_sort=True
        )
        self.given_mocked_grammar()
        self.given_generator()
        self.when_generate()
        self.then_no_common_examples_have_been_generated_for_answer_negation()



import iso8601
import urlparse
import mimetypes
import magic
import urllib

from tala.model.semantic_object import OntologySpecificSemanticObject, SemanticObject
from tala.model.image import Image
from tala.model.webview import Webview
from tala.model.date_time import DateTime

BOOLEAN = "boolean"
INTEGER = "integer"
DATETIME = "datetime"
REAL = "real"
STRING = "string"
IMAGE = "image"
DOMAIN = "domain"
WEBVIEW = "webview"


class Sort(SemanticObject):
    def __init__(self, name, dynamic=False):
        self._name = name
        self._dynamic = dynamic

    def get_name(self):
        return self._name

    @property
    def name(self):
        return self._name

    def is_dynamic(self):
        return self._dynamic

    def is_builtin(self):
        return False

    def is_boolean_sort(self):
        return self._name == BOOLEAN

    def is_datetime_sort(self):
        return self._name == DATETIME

    def is_real_sort(self):
        return self._name == REAL

    def is_integer_sort(self):
        return self._name == INTEGER

    def is_string_sort(self):
        return self._name == STRING

    def is_domain_sort(self):
        return self._name == DOMAIN

    def is_image_sort(self):
        return self._name == IMAGE

    def is_webview_sort(self):
        return self._name == WEBVIEW

    def __unicode__(self):
        return "Sort(%r, dynamic=%r)" % (self._name, self._dynamic)

    def __repr__(self):
        return "%s%s" % (self.__class__.__name__, (self._name, self._dynamic))

    def __eq__(self, other):
        try:
            return (other.get_name() == self.get_name() and other.is_dynamic() == self.is_dynamic())
        except AttributeError:
            return False

    def __ne__(self, other):
        return not (self == other)

    def __hash__(self):
        return hash((self.get_name(), self.is_dynamic()))


class BuiltinSort(Sort):
    def is_builtin(self):
        return True

    def normalize_value(self, value):
        return value


class InvalidValueException(Exception):
    pass


class RealSort(BuiltinSort):
    def __init__(self):
        BuiltinSort.__init__(self, REAL)

    def normalize_value(self, value):
        try:
            return float(value)
        except (ValueError, AttributeError, TypeError):
            raise InvalidValueException("Expected a real-number value but got '%s'." % value)


class IntegerSort(BuiltinSort):
    def __init__(self):
        BuiltinSort.__init__(self, INTEGER)

    def normalize_value(self, value):
        try:
            return int(value)
        except (ValueError, AttributeError, TypeError):
            raise InvalidValueException("Expected an integer value but got '%s'." % value)


class StringSort(BuiltinSort):
    def __init__(self):
        BuiltinSort.__init__(self, STRING)

    def normalize_value(self, value):
        if isinstance(value, basestring):
            return value
        else:
            raise InvalidValueException(
                "Expected a string value but got %s of type %s." % (value, value.__class__.__name__)
            )


class ImageSort(BuiltinSort):
    def __init__(self):
        BuiltinSort.__init__(self, IMAGE)

    def normalize_value(self, value):
        if isinstance(value, Image):
            if isinstance(value.url,
                          (str, unicode)) and "http" in value.url and self._has_mime_type(value.url, "image"):
                return value
            else:
                raise InvalidValueException("Expected an image URL but got '%s'." % value.url)
        else:
            raise InvalidValueException("Expected an image object but got '%s'." % value)

    def _has_mime_type(self, value, expected_mime_type):
        mime_type = self._get_mime_type_from_url(value)
        if mime_type:
            return mime_type.startswith(expected_mime_type + "/")
        else:
            return False

    def _get_mime_type_from_url(self, url):
        path = urlparse.urlparse(url).path
        mime_type, mime_subtype = mimetypes.guess_type(path)
        if not mime_type:
            try:
                filepath, _ = urllib.urlretrieve(url)
            except IOError:
                return None
            print "filepath=", filepath
            mime_type = magic.from_file(filepath, mime=True)
        return mime_type


class WebviewSort(BuiltinSort):
    def __init__(self):
        BuiltinSort.__init__(self, WEBVIEW)

    def normalize_value(self, value):
        if isinstance(value, Webview):
            if isinstance(value.url, (str, unicode)) and "http" in value.url:
                return value
            else:
                raise InvalidValueException("Expected a webview URL but got '%s'." % value.url)
        else:
            raise InvalidValueException("Expected a webview object but got '%s'." % value)


class BooleanSort(BuiltinSort):
    def __init__(self):
        BuiltinSort.__init__(self, BOOLEAN)

    def normalize_value(self, value):
        if value is True or value is False:
            return value
        else:
            raise InvalidValueException("Expected a boolean value but got '%s'." % value)


class DomainSort(BuiltinSort):
    def __init__(self):
        BuiltinSort.__init__(self, DOMAIN)


class DateTimeSort(BuiltinSort):
    def __init__(self):
        BuiltinSort.__init__(self, DATETIME)

    def normalize_value(self, value):
        if isinstance(value, DateTime):
            try:
                iso8601.parse_date(value.iso8601_string)
            except iso8601.ParseError:
                raise InvalidValueException(
                    "Expected a datetime value in ISO 8601 format but got '%s'." % value.iso8601_string
                )
            return value
        else:
            raise InvalidValueException("Expected a datetime object but got '%s'." % value)


class CustomSort(Sort, OntologySpecificSemanticObject):
    def __init__(self, ontology_name, name, dynamic=False):
        Sort.__init__(self, name, dynamic=dynamic)
        OntologySpecificSemanticObject.__init__(self, ontology_name)

    def __eq__(self, other):
        try:
            return (Sort.__eq__(self, other) and self.ontology_name == other.ontology_name)
        except AttributeError:
            return False

    def __hash__(self):
        return hash((self.ontology_name, Sort.__hash__(self)))


class BuiltinSortRepository(object):
    _repository = {
        BOOLEAN: BooleanSort(),
        INTEGER: IntegerSort(),
        DATETIME: DateTimeSort(),
        REAL: RealSort(),
        STRING: StringSort(),
        IMAGE: ImageSort(),
        DOMAIN: DomainSort(),
        WEBVIEW: WebviewSort()
    }

    @classmethod
    def has_sort(cls, name):
        return name in cls._repository

    @classmethod
    def get_sort(cls, name):
        if cls.has_sort(name):
            return cls._repository[name]
        else:
            raise UndefinedSort("Expected a built-in sort but got '%s'." % name)


class UndefinedSort(Exception):
    pass



import unittest

from tala.model.stack import StackSet, StackError, Stack
from tala.testing.utils import EqualityAssertionTestCaseMixin


class MockElement:
    def __init__(self, name):
        self._name = name

    def __unicode__(self):
        return "MockElement(%r)" % self._name


class StackTester(EqualityAssertionTestCaseMixin):
    def test_stacking(self):
        stack = self.create_stack()
        stack.push("first")
        stack.push("second")
        self.assertEquals(stack.pop(), "second")
        self.assertEquals(stack.pop(), "first")
        self.assertEquals(len(stack), 0)

    def test_initialize_with_content(self):
        stack = self.create_stack(["first", "second"])
        self.assertEquals(stack.pop(), "second")
        self.assertEquals(stack.pop(), "first")

    def test_equality_based_on_contents(self):
        stack1 = self.create_stack()
        stack1.push("first")
        stack1.push("second")
        stack2 = self.create_stack()
        stack2.push("first")
        stack2.push("second")
        self.assert_eq_returns_true_and_ne_returns_false_symmetrically(stack1, stack2)

    def test_top_on_empty_stack_raises_exception(self):
        stack = self.create_stack()
        self.assertRaises(StackError, stack.top)

    def test_top_returns_latest_stacked_element(self):
        stack = self.create_stack()
        stack.push("harry")
        element = "kalle"
        stack.push(element)
        self.assertEquals(element, stack.top())

    def test_newly_created_stack_is_empty(self):
        stack = self.create_stack()
        self.assertTrue(stack.isEmpty())

    def test_non_empty_stack_is_not_empty(self):
        stack = self.create_stack()
        stack.push("randomString")
        self.assertFalse(stack.isEmpty())

    def test_empty_stack_not_equals_none(self):
        empty_stack = self.create_stack()
        self.assertNotEquals(None, empty_stack)

    def test_pop_on_empty_stack_raises_exception(self):
        stack = self.create_stack()
        self.assertRaises(StackError, stack.pop)

    def test_isTop_succeeds_for_latest_stacked_element(self):
        stack = self.create_stack()
        stack.push("harry")
        element = "kalle"
        stack.push(element)
        self.assertTrue(stack.isTop(element))

    def test_isTop_on_empty_stack_returns_false(self):
        stack = self.create_stack()
        element = "kalle"
        self.assertFalse(stack.isTop(element))

    def test_isTop_for_non_top_element_returns_false(self):
        stack = self.create_stack()
        topElement = "kalle"
        stack.push(topElement)
        testElement = "nisse"
        self.assertFalse(stack.isTop(testElement))

    def test_iteration(self):
        stack = self.create_stack()
        stack.push("bottom")
        stack.push("top")
        list = []
        for element in stack:
            list.append(element)
        self.assertEquals(["top", "bottom"], list)

    def test_pushing_elem_of_correct_type_raises_no_exception(self):
        stack = self.create_stack(contentclass=str)
        stack.push("foo")

    def test_pushing_elem_of_wrong_type_raises_exception(self):
        stack = self.create_stack(contentclass=int)
        with self.assertRaises(TypeError):
            stack.push("foo")

    def test_delete_element(self):
        stack = self.create_stack()
        stack.push("first")
        stack.push("second")
        stack.remove("first")
        self.assertTrue("first" not in stack)
        self.assertTrue("second" in stack)

    def test_pushing_stack_on_stack(self):
        lower_stack = self.create_stack()
        lower_stack.push("bottom")
        lower_stack.push("second_from_bottom")
        upper_stack = self.create_stack()
        upper_stack.push("second_from_top")
        upper_stack.push("top")
        expected_stack = self.create_stack()
        expected_stack.push("bottom")
        expected_stack.push("second_from_bottom")
        expected_stack.push("second_from_top")
        expected_stack.push("top")
        result_stack = lower_stack
        result_stack.push_stack(upper_stack)
        self.assertEquals(expected_stack, result_stack)


class TestStacks(unittest.TestCase, StackTester):
    def setUp(self):
        self.create_stack = Stack

    def test_string_representation(self):
        stack = Stack()
        stack.push(MockElement("bottom"))
        stack.push(MockElement("top"))
        self.assertEquals("Stack([MockElement('top'), MockElement('bottom')])", unicode(stack))


class TestStackSets(unittest.TestCase, StackTester):
    def setUp(self):
        self.create_stack = StackSet

    def test_string_representation(self):
        stack = StackSet()
        stack.push(MockElement("bottom"))
        stack.push(MockElement("top"))
        self.assertEquals("stackset([MockElement('top'), MockElement('bottom')])", unicode(stack))

    def test_stacking_two_identical_elements_leaves_only_one(self):
        stack = StackSet()
        stack.push("first")
        stack.push("first")
        self.assertEquals(1, len(stack))

    def test_stacking_an_element_a_second_time_leaves_it_at_top(self):
        stack = StackSet()
        stack.push("first")
        stack.push("second")
        stack.push("first")
        expected_stack = StackSet()
        expected_stack.push("second")
        expected_stack.push("first")

        self.assertEquals(expected_stack, stack)

    def test_remove_if_exists_for_existing_element(self):
        set = StackSet(["first", "second"])
        set.remove_if_exists("first")
        expected_result = StackSet(["second"])
        self.assertEquals(expected_result, set)

    def test_remove_if_exists_for_non_existing_element_has_no_effect(self):
        set = StackSet(["first", "second"])
        set.remove_if_exists("third")
        expected_result = StackSet(["first", "second"])
        self.assertEquals(expected_result, set)


class TestStackSetView(unittest.TestCase):
    def test_get_view_of_stack_set(self):
        self._given_a_stack_set([0, 1, 2])
        self._given_a_ONE_pass_filter()
        self._given_a_filtered_view_of_stack()
        self._when_peeking_top_of_view()
        self._then_return_value_is(1)

    def _given_a_stack_set(self, integers):
        self._stack_set = StackSet(integers)

    def _given_a_ONE_pass_filter(self):
        self._filter = lambda integer: integer == 1

    def _given_a_filtered_view_of_stack(self):
        self._view = self._stack_set.create_view(self._filter)

    def _when_peeking_top_of_view(self):
        self._result = self._view.top()

    def _then_return_value_is(self, expected_return_value):
        self.assertEquals(expected_return_value, self._result)

    def test_push_in_view_simple_case(self):
        self._given_a_stack_set([])
        self._given_a_positive_filter()
        self._given_a_filtered_view_of_stack()
        self._when_pushing_an_element_in_view()
        self._then_the_object_becomes_locally_and_globally_visible()

    def _given_a_positive_filter(self):
        self._filter = lambda element: element > 0

    def _when_pushing_an_element_in_view(self):
        self._view.push(42)

    def _then_the_object_becomes_locally_and_globally_visible(self):
        self.assertIn(42, self._stack_set)
        self.assertIn(42, self._view)

    def test_pushing_item_already_on_top_of_view_does_not_modify_stack(self):
        self._given_a_stack_set([0, 1, 2])
        self._given_a_ONE_pass_filter()
        self._given_a_filtered_view_of_stack()
        self._when_pushing_an_element_already_on_top_of_view()
        self._then_the_stack_is_unmodified()

    def _when_pushing_an_element_already_on_top_of_view(self):
        self._view.push(1)

    def _then_the_stack_is_unmodified(self):
        self.assertEquals(2, self._stack_set.top())

    def test_pop_in_view(self):
        self._given_a_stack_set([2, 1, 0, -1])
        self._given_a_positive_filter()
        self._given_a_filtered_view_of_stack()
        self._when_popping_an_element_in_view()
        self._then_the_object_is_locally_and_globally_removed()

    def _when_popping_an_element_in_view(self):
        self._view.pop()

    def _then_the_object_is_locally_and_globally_removed(self):
        self.assertNotIn(1, self._stack_set)
        self.assertNotIn(1, self._view)

    def test_iterate_in_view(self):
        self._given_a_stack_set([2, 1, 0, -1])
        self._given_a_positive_filter()
        self._given_a_filtered_view_of_stack()
        self._when_iteratively_generating_a_list_from_view()
        self._then_the_list_is([1, 2])

    def _when_iteratively_generating_a_list_from_view(self):
        self._integer_list = list(self._view.__iter__())

    def _then_the_list_is(self, elems):
        self.assertEquals(elems, self._integer_list)

    def test_length_of_view(self):
        self._given_a_stack_set([2, 1, 0, -1])
        self._given_a_positive_filter()
        self._given_a_filtered_view_of_stack()
        self._then_length_is(len([1, 2]))

    def _then_length_is(self, length):
        self.assertEquals(length, len(self._view))



class InputHypothesis(object):
    def __init__(self, utterance, confidence):
        self._utterance = utterance
        self._confidence = confidence

    @property
    def utterance(self):
        return self._utterance

    @property
    def confidence(self):
        return self._confidence

    def __eq__(self, other):
        return self.utterance == other.utterance and self.confidence == other.confidence

    def __ne__(self, other):
        return not (self == other)

    def __unicode__(self):
        return "InputHypothesis(%r, %r)" % (self._utterance, self._confidence)

    def __str__(self):
        return unicode(self).encode("utf-8")

    def __repr__(self):
        return str(self)



import os
import os.path
import shutil
import tempfile
import unittest

from mock import Mock, patch

from tala.backend.dependencies.for_generating import BackendDependenciesForGenerating
from tala.ddd.building.steps.abstract_build_step import AbstractBuildStep
import tala.ddd.building.ddd_builder_for_generating
from tala.ddd.building.ddd_builder_for_generating import DDDBuilderForGenerating
from tala.model.ddd import DDD


class TestDDDBuilderForGenerating(unittest.TestCase):
    def setUp(self):
        self._temp_dir = tempfile.mkdtemp(prefix="TestDDDBuilderForGenerating")
        self._cwd = os.getcwd()
        os.chdir(self._temp_dir)
        self._asr = None
        self._use_word_list_correction = False
        self._mocked_build_step = None
        self._mock_ddd = None
        self._ddd_name = None
        self._mocked_backend_dependencies = None
        self._language_codes = []
        self._ddds_builder = None

    def tearDown(self):
        os.chdir(self._cwd)
        shutil.rmtree(self._temp_dir)

    def _given_mocked_ddd(self, is_rasa_enabled):
        mock_ddd = Mock(spec=DDD)
        mock_ddd.name = self._ddd_name
        mock_ddd.is_rasa_enabled = is_rasa_enabled
        self._mock_ddd = mock_ddd

    def _given_ddd_name(self, name):
        self._ddd_name = name

    def _given_file_exists(self, path):
        self._ensure_dir_exists(os.path.dirname(path))
        with open(path, "w"):
            pass

    def _ensure_dir_exists(self, path):
        if not os.path.exists(path):
            os.makedirs(path)

    def _given_language_codes(self, language_codes):
        self._language_codes = language_codes

    def _when_build_is_called(self):
        self._ddds_builder.build()

    def _given_mocked_backend_dependencies(self):
        backend_dependencies = Mock(spec=BackendDependenciesForGenerating)
        backend_dependencies.ddds = [self._mock_ddd]
        backend_dependencies.asr = self._asr
        backend_dependencies.use_word_list_correction = self._use_word_list_correction
        self._mocked_backend_dependencies = backend_dependencies

    def _given_ddds_builder_created(self):
        self._ddds_builder = DDDBuilderForGenerating(
            self._mocked_backend_dependencies, language_codes=self._language_codes
        )

    def _given_asr(self, asr):
        self._asr = asr

    @patch("{}.VerifyStepFactory".format(tala.ddd.building.ddd_builder_for_generating.__name__), autospec=True)
    def test_verify_invokes_build_on_verification_step(self, MockVerifyStepFactory):
        self._given_step_factory_creates_mocked_step(MockVerifyStepFactory)
        self._given_ddd_name("app")
        self._given_asr("none")
        self._given_mocked_ddd(is_rasa_enabled=False)
        self._given_mocked_backend_dependencies()
        self._given_file_exists("app/grammar/grammar_eng.py")
        self._given_language_codes(["eng"])
        self._given_ddds_builder_created()
        self._when_calling_verify()
        self._then_build_is_invoked_on_step()

    def _given_step_factory_creates_mocked_step(self, MockStepFactory):
        self._mocked_build_step = Mock(spec=AbstractBuildStep)
        mocked_factory = MockStepFactory.return_value
        mocked_factory.create.return_value = self._mocked_build_step

    def _when_calling_verify(self):
        self._ddds_builder.verify()

    def _then_build_is_invoked_on_step(self):
        self._mocked_build_step.build.assert_called_once()

    @patch("{}.GenerateStepFactory".format(tala.ddd.building.ddd_builder_for_generating.__name__), autospec=True)
    def test_build_invokes_build_on_generate_step(self, MockGenerateStepFactory):
        self._given_step_factory_creates_mocked_step(MockGenerateStepFactory)
        self._given_ddd_name("app")
        self._given_asr("none")
        self._given_mocked_ddd(is_rasa_enabled=False)
        self._given_mocked_backend_dependencies()
        self._given_file_exists("app/grammar/grammar_eng.py")
        self._given_language_codes(["eng"])
        self._given_ddds_builder_created()
        self._when_build_is_called()
        self._then_build_is_invoked_on_step()

    @patch("{}.GenerateStepFactory".format(tala.ddd.building.ddd_builder_for_generating.__name__), autospec=True)
    @patch("{}.CleanStepFactory".format(tala.ddd.building.ddd_builder_for_generating.__name__), autospec=True)
    def test_build_invokes_build_on_clean_step(self, MockCleanStepFactory, MockGenerateStepFactory):
        self._given_mocked_clean_step(MockCleanStepFactory)
        self._given_ddd_name("app")
        self._given_asr("none")
        self._given_mocked_ddd(is_rasa_enabled=False)
        self._given_mocked_backend_dependencies()
        self._given_file_exists("app/grammar/grammar_eng.py")
        self._given_language_codes(["eng"])
        self._given_ddds_builder_created()
        self._when_build_is_called()
        self._then_build_is_invoked_on_step()

    def _given_mocked_clean_step(self, MockCleanStepFactory):
        self._mocked_build_step = MockCleanStepFactory.return_value.create.return_value



from tala.model.ontology import Ontology, OntologyError
from tala.model.predicate import Predicate
from tala.model.sort import RealSort, IntegerSort, StringSort, CustomSort, ImageSort, WebviewSort, DateTimeSort
from tala.model.image import Image
from tala.model.webview import Webview
from tala.model.date_time import DateTime
from tala.model.test.utils import LibTestCase


class IndividualTestBase(LibTestCase):
    def setUp(self):
        self._create_ontology()
        self._create_semantic_objects()

    def _create_ontology(self):
        self.ontology_name = "mockup_ontology"
        self._city_sort = CustomSort(self.ontology_name, "city")
        sorts = {
            self._city_sort,
        }
        predicates = {
            self._create_predicate("price", RealSort()),
            self._create_predicate("number_of_passengers", IntegerSort()),
            self._create_predicate("map_to_show", ImageSort()),
            self._create_predicate("map_to_browse", WebviewSort()),
            self._create_predicate("departure_time", DateTimeSort()),
            self._create_predicate("comment", StringSort()),
            self._create_predicate("selected_city", self._city_sort),
        }
        individuals = {
            "paris": CustomSort(self.ontology_name, "city"),
            "london": CustomSort(self.ontology_name, "city"),
        }
        actions = set([])
        self.ontology = Ontology(self.ontology_name, sorts, predicates, individuals, actions)

        self.empty_ontology = Ontology("empty_ontology", {}, {}, {}, set([]))

    def _create_predicate(self, *args, **kwargs):
        return Predicate(self.ontology_name, *args, **kwargs)

    def _create_semantic_objects(self):
        self.sort_city = self.ontology.get_sort("city")
        self.individual_paris = self.ontology.create_individual("paris")
        self.individual_not_paris = self.ontology.create_negative_individual("paris")
        self.integer_individual = self.ontology.create_individual(1234, IntegerSort())
        self.string_individual = self.ontology.create_individual("a string")
        self.image_individual = self.ontology.create_individual(Image("http://mymap.com/map.png"))
        self.webview_individual = self.ontology.create_individual(Webview("http://mymap.com/map.html"))
        self.datetime_individual = self.ontology.create_individual(DateTime("2018-04-11T22:00:00.000Z"))


class IndividualTest(IndividualTestBase):
    def test_create_real_individual_with_ontology_with_real_type(self):
        self.ontology.create_individual(123.50)

    def test_create_real_individual_with_ontology_without_real_type(self):
        ontology_without_real_type = self.empty_ontology
        with self.assertRaises(OntologyError):
            ontology_without_real_type.create_individual(123.50)

    def test_create_integer_individual_with_ontology_with_integer_type(self):
        self.ontology.create_individual(123)

    def test_create_integer_individual_with_ontology_without_integer_type(self):
        with self.assertRaises(OntologyError):
            ontology_without_integer_type = self.empty_ontology
            ontology_without_integer_type.create_individual(123)

    def test_real_individual_unicode(self):
        real_individual = self.ontology.create_individual(1234.0)
        self.assertEquals("1234.0", unicode(real_individual))

    def test_integer_individual_unicode(self):
        self.assertEquals("1234", unicode(self.integer_individual))

    def test_string_individual_unicode(self):
        self.assertEquals('"a string"', unicode(self.string_individual))

    def test_image_individual_unicode(self):
        self.assertEquals('image("http://mymap.com/map.png")', unicode(self.image_individual))

    def test_webview_individual_unicode(self):
        self.assertEquals(u'webview("http://mymap.com/map.html")', unicode(self.webview_individual))

    def test_datetime_individual_unicode(self):
        self.assertEquals('datetime(2018-04-11T22:00:00.000Z)', unicode(self.datetime_individual))

    def test_create_individual(self):
        individual = self.ontology.create_individual("paris")
        self.assertEquals("paris", individual.getValue())

    def test_create_negative_individual(self):
        individual = self.ontology.create_negative_individual("paris")
        self.assertEquals("paris", individual.getValue())

    def test_create_invalid_individual(self):
        ontology_without_string_predicate = self.empty_ontology
        with (self.assertRaises(OntologyError)):
            ontology_without_string_predicate.create_individual("kalle")

    def test_create_invalid_negative_individual(self):
        ontology_without_string_predicate = self.empty_ontology
        with (self.assertRaises(OntologyError)):
            ontology_without_string_predicate.create_negative_individual("kalle")

    def test_get_sort(self):
        individual = self.ontology.create_individual("paris")
        self.assertEquals(self.sort_city, individual.getSort())

    def test_equality(self):
        individual1 = self.ontology.create_individual("paris")
        individual2 = self.ontology.create_individual("paris")
        self.assert_eq_returns_true_and_ne_returns_false_symmetrically(individual1, individual2)

    def test_non_equality_due_to_value(self):
        other_individual = self.ontology.create_individual("london")
        self.assert_eq_returns_false_and_ne_returns_true_symmetrically(self.individual_paris, other_individual)

    def test_non_equality_due_to_sort(self):
        other_individual = self.ontology.create_individual("paris", sort=StringSort())
        self.assert_eq_returns_false_and_ne_returns_true_symmetrically(self.individual_paris, other_individual)

    def test_non_equality_due_to_polarity(self):
        positive_individual = self.ontology.create_individual("paris")
        negative_individual = self.ontology.create_negative_individual("paris")
        self.assertNotEqual(positive_individual, negative_individual)
        self.assertNotEqual(negative_individual, positive_individual)

    def test_non_equality_with_non_individual(self):
        non_individual = "paris"
        self.assert_eq_returns_false_and_ne_returns_true_symmetrically(self.individual_paris, non_individual)

    def test_unicode_for_individual_of_custom_sort(self):
        self.assertEquals("paris", unicode(self.individual_paris))

    def test_unicode_for_negative_individual_of_custom_sort(self):
        self.assertEquals("~paris", unicode(self.individual_not_paris))

    def test_individual_is_positive(self):
        self.assertTrue(self.individual_paris.is_positive())

    def test_negative_individual_is_not_positive(self):
        self.assertFalse(self.individual_not_paris.is_positive())

    def test_hashing(self):
        s = set()
        s.add(self.individual_paris)

    def test_negate_positive_individual(self):
        self.assertFalse(self.individual_paris.negate().is_positive())

    def test_negate_negative_individual(self):
        self.assertTrue(self.individual_not_paris.negate().is_positive())


class ValueAsJsonObjectTestCase(IndividualTestBase):
    def test_custom_sort(self):
        self.when_get_value_as_json_object(self.individual_paris)
        self.then_result_is({"value": "paris"})

    def when_get_value_as_json_object(self, proposition):
        self._actual_result = proposition.value_as_json_object()

    def test_string(self):
        self.when_get_value_as_json_object(self.string_individual)
        self.then_result_is({"value": "a string"})

    def test_image(self):
        self.when_get_value_as_json_object(self.image_individual)
        self.then_result_is({"value": "http://mymap.com/map.png"})

    def test_webview(self):
        self.when_get_value_as_json_object(self.webview_individual)
        self.then_result_is({"value": "http://mymap.com/map.html"})

    def test_datetime(self):
        self.when_get_value_as_json_object(self.datetime_individual)
        self.then_result_is({"value": "2018-04-11T22:00:00.000Z"})


class IsIndividualTests(LibTestCase):
    def setUp(self):
        self.setUpLibTestCase()

    def test_is_individual_false(self):
        self.assertFalse(self.proposition_dest_city_paris.is_individual())

    def test_is_individual_true(self):
        self.assertTrue(self.individual_paris.is_individual())



import unittest

from tala.model.plan import Plan, UnableToDetermineOntologyException
from tala.model.plan_item import IfThenElse
from tala.model.question_raising_plan_item import FindoutPlanItem, RaisePlanItem
from tala.model.semantic_object import OntologySpecificSemanticObject
from tala.model.test.utils import LibTestCase


class PlanTests(LibTestCase):
    def setUp(self):
        self.setUpLibTestCase()
        self.findout_price = FindoutPlanItem(self.domain_name, self.price_question)
        self.findout_dest_city = FindoutPlanItem(self.domain_name, self.dest_city_question)
        self.consequent = RaisePlanItem(self.domain_name, self.price_question)
        self.alternative = RaisePlanItem(self.domain_name, self.dest_city_question)
        self.if_then_else_item = IfThenElse("mockup_condition", self.consequent, self.alternative)

    def test_plan_iteration_steps_into_nested_blocks(self):
        self._given_plan_with_nested_item()
        expected_list = [self.findout_price, self.findout_dest_city, self.consequent, self.alternative]
        self.assertEquals(expected_list, list(self.plan))

    def _given_plan_with_nested_item(self):
        self.plan = Plan([self.if_then_else_item, self.findout_dest_city, self.findout_price])

    def test_plan_iteration_after_modification(self):
        plan = Plan()
        plan.push(self.findout_price)
        expected_list = [self.findout_price]
        self.assertEquals(expected_list, list(plan))

    def test_removal_of_findout_from_nested_block(self):
        self._given_plan_with_nested_item()
        self._when_consequent_is_removed()
        self._then_all_elements_but_consequent_are_left()

    def _when_consequent_is_removed(self):
        self.plan.remove(self.consequent)

    def _then_all_elements_but_consequent_are_left(self):
        expected_list = [self.findout_price, self.findout_dest_city, self.alternative]
        self.assertEquals(expected_list, list(self.plan))


class SemanticObjectPlanTests(unittest.TestCase):
    def setUp(self):
        self._semantic_objects = set()

    def test_ontology_name_with_ontology_specific_semantic_object(self):
        self._given_semantic_object_of_ontology("an ontology")
        self._given_plan()
        self._when_asking_for_ontology_name()
        self._then_ontology_name_is("an ontology")

    def _given_semantic_object_of_ontology(self, ontology):
        semantic_object = OntologySpecificSemanticObject(ontology)
        self._semantic_objects.add(semantic_object)

    def _given_plan(self):
        self._plan = Plan(self._semantic_objects)

    def _when_asking_for_ontology_name(self):
        self._result = self._plan.ontology_name

    def _then_ontology_name_is(self, expected_name):
        actual_name = self._result
        self.assertEqual(expected_name, actual_name)

    def test_ontology_name_with_multiple_ontology_specific_semantic_objects(self):
        self._given_semantic_object_of_ontology("an ontology")
        self._given_semantic_object_of_ontology("another ontology")
        self._given_plan()
        self._when_asking_for_ontology_name_then_exception_is_raised()

    def _when_asking_for_ontology_name_then_exception_is_raised(self):
        with self.assertRaises(UnableToDetermineOntologyException):
            self._result = self._plan.ontology_name

    def test_ontology_name_with_multiple_ontology_specific_semantic_objects_of_same_ontology(self):
        self._given_semantic_object_of_ontology("an ontology")
        self._given_semantic_object_of_ontology("an ontology")
        self._given_plan()
        self._when_asking_for_ontology_name()
        self._then_ontology_name_is("an ontology")



import unittest

from tala.utils import text_formatting


class TextFormattingTests(unittest.TestCase):
    def setUp(self):
        self._elements = None
        self._result = None

    def test_readable_list_with_no_elements(self):
        self.given_list([])
        self.when_creating_readable_list()
        self.then_result_was("")

    def given_list(self, elements):
        self._elements = elements

    def when_creating_readable_list(self):
        self._result = text_formatting.readable_list(self._elements)

    def then_result_was(self, expected):
        self.assertEqual(expected, self._result)

    def test_readable_list_with_one_element(self):
        self.given_list(["element"])
        self.when_creating_readable_list()
        self.then_result_was("element")

    def test_readable_list_with_two_elements(self):
        self.given_list(["first element", "second element"])
        self.when_creating_readable_list()
        self.then_result_was("first element and second element")

    def test_readable_list_with_three_elements(self):
        self.given_list(["first element", "second element", "third element"])
        self.when_creating_readable_list()
        self.then_result_was("first element, second element and third element")



import copy
import re

from tala.ddd.utils import CacheMethod
from tala.model.goal import HandleGoal, PerformGoal, ResolveGoal
from tala.model.individual import Yes, No
from tala.model.lambda_abstraction import LambdaAbstractedGoalProposition
from tala.model.speaker import Speaker
from tala.model.set import Set
from tala.model.move import ICMMove, IssueICMMove, ICMMoveWithSemanticContent, ReportMove, PrereportMove, GreetMove, QuitMove, MuteMove, UnmuteMove, AskMove, RequestMove, AnswerMove, ICMMoveWithStringContent
from tala.model.ontology import OntologyError
from tala.model.plan_item import AssumePlanItem, AssumeSharedPlanItem, AssumeIssuePlanItem, RespondPlanItem, DoPlanItem, BindPlanItem, ConsultDBPlanItem, JumpToPlanItem, IfThenElse, ForgetAllPlanItem, ForgetPlanItem, ForgetIssuePlanItem, InvokeServiceQueryPlanItem, InvokeServiceActionPlanItem
from tala.model.polarity import Polarity
from tala.model.proposition import GoalProposition, PropositionSet, ServiceActionStartedProposition, ServiceActionTerminatedProposition, ServiceResultProposition, ResolvednessProposition, PreconfirmationProposition, UnderstandingProposition, RejectedPropositions, PrereportProposition, PredicateProposition
from tala.model.question import AltQuestion, YesNoQuestion, WhQuestion
from tala.model.question_raising_plan_item import QuestionRaisingPlanItem, FindoutPlanItem, RaisePlanItem
from tala.model.service_action_outcome import SuccessfulServiceAction, FailedServiceAction


class ParseError(Exception):
    pass


class ParseFailure(Exception):
    pass


class Parser:
    def __init__(self, ddd_name, ontology, domain_name=None):
        self._ddd_name = ddd_name
        self.ontology = ontology
        self.ontology_name = ontology.get_name()
        self.domain_name = domain_name
        self._cache_method = CacheMethod(self, self._cacheable_parse)
        self._clauses = [
            self._parse_set,
            self._parse_decorated_non_icm_move,
            self._parse_non_icm_move,
            self._parse_findout_plan_item,
            self._parse_do_plan_item,
            self._parse_raise_plan_item,
            self._parse_bind_plan_item,
            self._parse_respond_plan_item,
            self._parse_consultDB_plan_item,
            self._parse_if_then_else_plan_item,
            self._parse_jumpto_plan_item,
            self._parse_forget_all_plan_item,
            self._parse_forget_plan_item,
            self._parse_assume_plan_item,
            self._parse_assume_shared_plan_item,
            self._parse_goal,
            self._parse_assume_issue_item,
            self._parse_forget_issue_plan_item,
            self._parse_invoke_service_query_plan_item,
            self._parse_deprecated_dev_query_plan_item,
            self._parse_invoke_service_action_plan_item,
            self._parse_deprecated_dev_perform_plan_item,
            self._parse_question,
            self._parse_lambda_abstracted_goal_proposition,
            self._parse_lambda_abstracted_predicate_proposition,
            self._parse_service_result_proposition,
            self._parse_successful_service_action,
            self._parse_failed_service_action,
            self._parse_decorated_icm_move,
            self._parse_icm_move,
            self._parse_proposition,
            self._parse_negative_individual,
            self._parse_yes_or_no,
            self._parse_prop_set,
            self._parse_action,
            self._parse_individual,
            self._parse_predicate,
            self._parse_string,
        ]

    def clear(self):
        self._cache_method.clear()

    def parse(self, string):
        object = self._cacheable_parse(string)
        new_instance = copy.deepcopy(object)
        return new_instance

    def _cacheable_parse(self, string):
        try:
            return self._parse(string)
        except OntologyError as exception:
            raise ParseError("failed to parse '%s': %s" % (string, exception))

    def parse_parameters(self, string):
        params = {}
        string = self._strip_curly_brackets(string)
        while self._more_data_to_read(string):
            (key, value, rest) = self._get_next_question_param(string)
            params[key] = value
            string = rest
        return params

    def _parse(self, string):
        for parse_method in self._clauses:
            try:
                return parse_method(string)
            except ParseFailure:
                pass
        raise ParseError("failed to parse '" + string + "' (ontology=" + unicode(self.ontology) + ")")

    def _parse_incrementally(self, string):
        for n in range(1, len(string) + 1):
            try:
                result = self._parse(string[0:n])
                if result:
                    return result, string[n:]
            except ParseError:
                pass

    def _parse_goal(self, string):
        try:
            return self._parse_resolve_goal(string)
        except ParseFailure:
            pass
        try:
            return self._parse_resolve_user_goal(string)
        except ParseFailure:
            pass
        try:
            return self._parse_perform_goal(string)
        except ParseFailure:
            pass
        try:
            return self._parse_handle_goal(string)
        except ParseFailure:
            pass
        raise ParseFailure()

    def _parse_decorated_non_icm_move(self, string):
        m = re.search('^Move\((.*)\)$', string)
        if m:
            move_and_maybe_realization_data_string = m.group(1)
            move, rest = self._parse_incrementally(move_and_maybe_realization_data_string)
            if rest:
                m = re.search('^, (.*)$', rest)
                if m:
                    realization_data_string = m.group(1)
                    realization_data = self._parse_realization_data(realization_data_string)
                    move.set_realization_data(**realization_data)
            return move
        else:
            raise ParseFailure()

    def _parse_non_icm_move(self, string):
        try:
            return self._parse_basic_move(string)
        except ParseFailure:
            pass
        try:
            return self._parse_request_move(string)
        except ParseFailure:
            pass
        try:
            return self._parse_ask_move(string)
        except ParseFailure:
            pass
        try:
            return self._parse_answer_move(string)
        except ParseFailure:
            pass
        try:
            return self._parse_report_move(string)
        except ParseFailure:
            pass
        try:
            return self._parse_prereport_move(string)
        except ParseFailure:
            pass
        raise ParseFailure()

    def _parse_basic_move(self, string):
        matcher = re.search('^(greet|mute|unmute|quit)$', string)
        if matcher:
            move_type_string = matcher.group(1)
            if move_type_string == "greet":
                return GreetMove()
            elif move_type_string == "mute":
                return MuteMove()
            elif move_type_string == "unmute":
                return UnmuteMove()
            elif move_type_string == "quit":
                return QuitMove()

        raise ParseFailure()

    def _parse_set(self, string):
        m = re.search('^\{(.*)\}$', string)
        if m:
            if m.group(1) == "":
                return Set()
            else:
                content_as_strings = m.group(1).split(", ")
                s = Set()
                for content_as_string in content_as_strings:
                    s.add(self.parse(content_as_string))
                return s
        else:
            raise ParseFailure()

    def _parse_string(self, string):
        m = re.search('^"[^"]*"$', string)
        if m:
            return string
        else:
            raise ParseFailure()

    def _parse_proposition(self, string):
        try:
            return self._parse_deprecated_service_action_terminated_proposition(string)
        except ParseFailure:
            pass
        try:
            return self._parse_deprecated_service_action_started_proposition(string)
        except ParseFailure:
            pass
        try:
            return self._parse_preconfirmation_proposition(string)
        except ParseFailure:
            pass
        try:
            return self._parse_prereport_proposition(string)
        except ParseFailure:
            pass
        try:
            return self._parse_goal_proposition(string)
        except ParseFailure:
            pass
        try:
            return self._parse_rejected_proposition(string)
        except ParseFailure:
            pass
        try:
            return self._parse_predicate_proposition(string)
        except ParseFailure:
            pass
        try:
            return self._parse_understanding_proposition(string)
        except ParseFailure:
            pass
        try:
            return self._parse_resolvedness_proposition(string)
        except ParseFailure:
            pass
        self._check_if_deprecated_action_proposition(string)
        self._check_if_deprecated_issue_proposition(string)
        raise ParseFailure()

    def _check_if_deprecated_action_proposition(self, string):
        m = re.search('^action\((\w+)\)$', string)
        if m:
            action_name = m.group(1)
            raise ParseError(
                "'action(%s)' is not a valid proposition. Perhaps you mean 'goal(perform(%s))'." %
                (action_name, action_name)
            )

    def _check_if_deprecated_issue_proposition(self, string):
        m = re.search('^issue\((.+)\)$', string)
        if m:
            issue_name = m.group(1)
            raise ParseError(
                "'issue(%s)' is not a valid proposition. Perhaps you mean 'goal(resolve(%s))'." %
                (issue_name, issue_name)
            )

    def _parse_preconfirmation_proposition(self, string):
        m = re.search('^(~?)preconfirmed\((?P<action>\w+), (\[[^\]]*\])\)$', string)
        if m:
            (polarity_str, action_value, parameter_string) = (m.group(1), m.group("action"), m.group(3))
            polarity = self._parse_polarity(polarity_str)
            parameter_list = self._parse_proposition_list(parameter_string)
            return PreconfirmationProposition(self.ontology_name, action_value, parameter_list, polarity)
        raise ParseFailure()

    def _parse_prereport_proposition(self, string):
        m = re.search('^prereported\((?P<action>\w+), (\[[^\]]*\])\)$', string)
        if m:
            (action_value, parameter_string) = (m.group("action"), m.group(2))
            parameter_list = self._parse_proposition_list(parameter_string)
            return PrereportProposition(self.ontology_name, action_value, parameter_list)
        raise ParseFailure()

    def _parse_report_move(self, string):
        m = re.search('^report\((.*)\)$', string)
        if m:
            report_proposition_string = m.group(1)
            report_proposition = self._parse(report_proposition_string)
            return ReportMove(report_proposition)
        raise ParseFailure()

    def _parse_service_result_proposition(self, string):
        m = re.search('^ServiceResultProposition\((\w+), (\[[^\]]*\]), (.+)\)$', string)
        if m:
            (action_value, parameter_string, action_outcome_string) = m.groups()
            parameter_list = self._parse_proposition_list(parameter_string)
            action_outcome = self._parse(action_outcome_string)
            return ServiceResultProposition(self.ontology_name, action_value, parameter_list, action_outcome)
        raise ParseFailure()

    def _parse_successful_service_action(self, string):
        if string == "SuccessfulServiceAction()":
            return SuccessfulServiceAction()
        raise ParseFailure()

    def _parse_failed_service_action(self, string):
        m = re.search('^FailedServiceAction\((\w+)\)$', string)
        if m:
            (failure_reason, ) = m.groups()
            return FailedServiceAction(failure_reason)
        raise ParseFailure()

    def _parse_prereport_move(self, string):
        m = re.search('^prereport\((\w+), (\[[^\]]*\])\)$', string)
        if m:
            (action_value, arguments_string) = m.groups()
            arguments_list = self._parse_proposition_list(arguments_string)
            return PrereportMove(self.ontology_name, action_value, arguments_list)
        raise ParseFailure()

    def _parse_ask_move(self, string):
        m = re.search('^ask\((.+)\)', string)
        if m:
            question_string = m.group(1)
            question = self._parse_question(question_string)
            return AskMove(question)
        else:
            raise ParseFailure()

    def _parse_question(self, string):
        m = re.search('^\?(.*)$', string)
        if m:
            question_content_string = m.group(1)
            question_content = self.parse(question_content_string)
            try:
                if question_content.is_lambda_abstracted_predicate_proposition():
                    return WhQuestion(question_content)
            except AttributeError:
                pass
            try:
                if question_content.is_lambda_abstracted_goal_proposition():
                    return WhQuestion(question_content)
            except AttributeError:
                pass
            try:
                if question_content.is_proposition_set():
                    return AltQuestion(question_content)
            except AttributeError:
                pass
            try:
                if question_content.is_proposition():
                    return YesNoQuestion(question_content)
            except AttributeError:
                pass
        raise ParseFailure()

    def _parse_list_of_questions(self, string):
        string_without_brackets = self._strip_brackets(string)
        question_strings = [str.strip() for str in string_without_brackets.split(",")]
        return [self._parse_question(question_string) for question_string in question_strings]

    def _parse_lambda_abstracted_goal_proposition(self, string):
        if string == "X.goal(X)":
            return LambdaAbstractedGoalProposition()
        else:
            raise ParseFailure()

    def _parse_lambda_abstracted_predicate_proposition(self, string):
        m = re.search('^X\.(.+)\(X\)$', string)
        if m:
            predicate_name = m.group(1)
            if predicate_name == "action":
                raise ParseError("'?X.action(X)' is not a valid question. Perhaps you mean '?X.goal(X)'.")
            predicate = self.ontology.get_predicate(predicate_name)
            return self.ontology.create_lambda_abstracted_predicate_proposition(predicate)
        else:
            raise ParseFailure()

    def _parse_answer_move(self, string):
        m = re.search('^answer\((.+)\)', string)
        if m:
            answer_string = m.group(1)
            answer = self.parse(answer_string)
            answer_move = AnswerMove(answer)
            return answer_move
        else:
            raise ParseFailure()

    def _parse_decorated_icm_move(self, string):
        m = re.search('^ICMMove\(([^,]+)((, *)(.+))?\)$', string)
        if m:
            icm_string = m.group(1)
            realization_data_string = m.group(4)
            icm = self._parse_icm_move(icm_string)
            if realization_data_string:
                realization_data = self._parse_realization_data(realization_data_string)
                icm.set_realization_data(**realization_data)
            return icm
        else:
            raise ParseFailure()

    def _parse_realization_data(self, string):
        realization_data = dict()
        while self._more_data_to_read(string):
            (key, value, rest) = self._get_next_realization_data(string)
            realization_data[key] = value
            string = rest

        return realization_data

    def _get_next_realization_data(self, string):
        m = re.search('^(?P<key>[^=]+)=(?P<value>[^=]+) *\, *(?P<rest>[^=]+=.*)$', string)
        if m:
            rest = m.group('rest')
        else:
            m = re.search('^(?P<key>[^=]+)=(?P<value>.+)$', string)
            rest = ""
        if m:
            key = m.group('key')
            value_as_string = m.group('value')
            if key == "speaker":
                value = value_as_string
            elif key == "perception_confidence":
                value = float(value_as_string)
            elif key == "understanding_confidence":
                value = float(value_as_string)
            elif key == "utterance" or key == "ddd_name":
                value = self._strip_quotes(value_as_string)
            else:
                raise ParseFailure("unsupported realization attribute '%s'" % key)
            return (key, value, rest)
        else:
            raise ParseFailure()

    def _strip_quotes(self, string):
        m = re.search('^"([^"]*)"$', string)
        if m:
            return m.group(1)
        else:
            m = re.search("^'([^']*)'$", string)
            if m:
                return m.group(1)
            else:
                raise ParseFailure()

    def _parse_icm_move(self, string):
        m = re.search('^icm:', string)
        if m:
            try:
                return self._parse_reraise_or_accommodate_or_resume_icm(string)
            except ParseFailure:
                pass
            try:
                return self._parse_loadplan_icm(string)
            except ParseFailure:
                pass
            try:
                return self._parse_perception_or_acceptance_icm(string)
            except ParseFailure:
                pass
            try:
                return self._parse_understanding_icm(string)
            except ParseFailure:
                pass
        raise ParseFailure()

    def _parse_reraise_or_accommodate_or_resume_icm(self, string):
        matcher = re.search(
            '^icm:(%s|%s|%s)(:([^:]+))?$' % (ICMMove.RERAISE, ICMMove.ACCOMMODATE, ICMMove.RESUME), string
        )
        if matcher:
            (icm_type, content_string) = (matcher.group(1), matcher.group(3))
            if content_string:
                content = self.parse(content_string)
                return ICMMoveWithSemanticContent(icm_type, content)
            return ICMMove(icm_type)
        else:
            raise ParseFailure()

    def _parse_loadplan_icm(self, string):
        matcher = re.search('^icm:loadplan', string)
        if matcher:
            return ICMMove(ICMMove.LOADPLAN)
        else:
            raise ParseFailure()

    def _parse_perception_or_acceptance_icm(self, string):
        try:
            return self._parse_contentless_perception_or_acceptance_icm(string)
        except ParseFailure:
            pass
        try:
            return self._parse_contentful_perception_icm(string)
        except ParseFailure:
            pass
        try:
            return self._parse_contentful_acceptance_icm(string)
        except ParseFailure:
            pass
        raise ParseFailure()

    def _parse_contentless_perception_or_acceptance_icm(self, string):
        m = re.search('^icm:(per|acc)\*(pos|neg|int)$', string)
        if m:
            icm_type = m.group(1)
            polarity = m.group(2)
            return ICMMove(icm_type, polarity=polarity)
        raise ParseFailure()

    def _parse_contentful_perception_icm(self, string):
        m = re.search('^icm:per\*(pos|neg|int):"([^"]*)"$', string)
        if m:
            (polarity, content_string) = m.groups()
            return ICMMoveWithStringContent(ICMMove.PER, content_string, polarity=polarity)
        raise ParseFailure()

    def _parse_contentful_acceptance_icm(self, string):
        m = re.search('^icm:(per|acc)\*(pos|neg|int)(:([^:]*))?$', string)
        if m:
            (icm_type, polarity, junk, content_string) = m.groups()
            if content_string == "issue":
                return IssueICMMove(icm_type, polarity=polarity)
            else:
                content = self.parse(content_string)
            return ICMMoveWithSemanticContent(type=icm_type, content=content, polarity=polarity)
        raise ParseFailure()

    def _parse_understanding_icm(self, string):
        m = re.search('^icm:(sem|und)\*(int|pos|neg)(:((.+)\*)?([^:]+))?$', string)
        if m:
            (type, polarity, foo, foo, content_speaker, content_string) = m.groups()
            if content_string:
                content = self.parse(content_string)
            else:
                content = None
            return ICMMoveWithSemanticContent(type, content, content_speaker=content_speaker, polarity=polarity)
        else:
            raise ParseFailure()

    def _parse_resolvedness_proposition(self, string):
        m = re.search('^resolved\((.+)\)$', string)
        if m:
            issue_string = m.group(1)
            issue = self.parse(issue_string)
            return ResolvednessProposition(issue)
        else:
            raise ParseFailure()

    def _parse_request_move(self, string):
        matcher = re.search('^request\(([^\{\}]+)\)(.*)', string)
        if matcher:
            action_string = matcher.group(1)
            action = self.parse(action_string)
            return RequestMove(action)
        else:
            raise ParseFailure()

    def _parse_findout_plan_item(self, string):
        m = re.search('^findout\((.*)\)$', string)
        if m:
            question_string = m.group(1)
            question = self._parse_question(question_string)
            return FindoutPlanItem(self.domain_name, question)
        else:
            raise ParseFailure()

    def _parse_do_plan_item(self, string):
        m = re.search('^do\((\w+)\)$', string)
        if m:
            action_string = m.group(1)
            action = self.parse(action_string)
            return DoPlanItem(action)
        else:
            raise ParseFailure()

    def _parse_bind_plan_item(self, string):
        m = re.search('^bind\((.+)\)$', string)
        if m:
            question_string = m.group(1)
            question = self._parse_question(question_string)
            return BindPlanItem(question)
        else:
            raise ParseFailure()

    def _parse_forget_all_plan_item(self, string):
        if string == "forget_all":
            return ForgetAllPlanItem()
        else:
            raise ParseFailure()

    def _parse_forget_plan_item(self, string):
        m = re.search('^forget\((.+)\)$', string)
        if m:
            proposition_string = m.group(1)
            proposition = self.parse(proposition_string)
            return ForgetPlanItem(proposition)
        else:
            raise ParseFailure()

    def _parse_assume_plan_item(self, string):
        m = re.search('^assume\((.+)\)$', string)
        if m:
            proposition_string = m.group(1)
            proposition = self._parse_proposition(proposition_string)
            return AssumePlanItem(proposition)
        else:
            raise ParseFailure()

    def _parse_assume_shared_plan_item(self, string):
        m = re.search('^assume_shared\((.+)\)$', string)
        if m:
            proposition_string = m.group(1)
            proposition = self._parse_proposition(proposition_string)
            return AssumeSharedPlanItem(proposition)
        else:
            raise ParseFailure()

    def _parse_assume_issue_item(self, string):
        m = re.search('^assume_issue\((.+)\)$', string)
        if m:
            issue_string = m.group(1)
            issue = self.parse(issue_string)
            return AssumeIssuePlanItem(issue)
        else:
            raise ParseFailure()

    def _parse_forget_issue_plan_item(self, string):
        m = re.search('^forget_issue\((.+)\)$', string)
        if m:
            issue_string = m.group(1)
            issue = self.parse(issue_string)
            return ForgetIssuePlanItem(issue)
        else:
            raise ParseFailure()

    def _parse_invoke_service_query_plan_item(self, string):
        m = re.search('^invoke_service_query\(([^,]+)\)$', string)
        if m:
            (issue_string, ) = m.groups()
            issue = self.parse(issue_string)
            return InvokeServiceQueryPlanItem(issue, min_results=1, max_results=1)
        else:
            raise ParseFailure()

    def _parse_deprecated_dev_query_plan_item(self, string):
        m = re.search('^dev_query\(([^,]+)\)$', string)
        if m:
            (issue_string, ) = m.groups()
            issue = self.parse(issue_string)
            return InvokeServiceQueryPlanItem(issue, min_results=1, max_results=1)
        else:
            raise ParseFailure()

    def _parse_invoke_service_action_plan_item(self, string):
        m = re.search('^invoke_service_action\(([^,]+), *(\{.*\})\)$', string)
        if m:
            (service_action, params_string) = m.groups()
            params = self._parse_invoke_service_action_params(params_string)
            return InvokeServiceActionPlanItem(self.ontology.name, service_action, **params)
        else:
            raise ParseFailure()

    def _parse_deprecated_dev_perform_plan_item(self, string):
        m = re.search('^dev_perform\(([^,]+), *(\{.*\})\)$', string)
        if m:
            (service_action, params_string) = m.groups()
            params = self._parse_invoke_service_action_params(params_string)
            return InvokeServiceActionPlanItem(self.ontology.name, service_action, **params)
        else:
            raise ParseFailure()

    def _parse_raise_plan_item(self, string):
        m = re.search('^raise\((.+)\)$', string)
        if m:
            question_string = m.group(1)
            question = self._parse_question(question_string)
            return RaisePlanItem(self.domain_name, question)
        else:
            raise ParseFailure()

    def _parse_respond_plan_item(self, string):
        m = re.search('^respond\((.+)\)$', string)
        if m:
            question_string = m.group(1)
            question = self._parse_question(question_string)
            return RespondPlanItem(question)
        else:
            raise ParseFailure()

    def _parse_resolve_goal(self, string):
        m = re.search('^resolve\((.+)\)$', string)
        if m:
            question_string = m.group(1)
            question = self._parse_question(question_string)
            return ResolveGoal(question, Speaker.SYS)
        else:
            raise ParseFailure()

    def _parse_resolve_user_goal(self, string):
        m = re.search('^resolve_user\((.+)\)$', string)
        if m:
            question_string = m.group(1)
            question = self._parse_question(question_string)
            return ResolveGoal(question, Speaker.USR)
        else:
            raise ParseFailure()

    def _parse_perform_goal(self, string):
        m = re.search('^perform\((.+)\)$', string)
        if m:
            action_string = m.group(1)
            action = self._parse_action(action_string)
            return PerformGoal(action)
        else:
            raise ParseFailure()

    def _parse_handle_goal(self, string):
        m = re.search('^handle\((.+)\)$', string)
        if m:
            service_action = m.group(1)
            return HandleGoal(self.ontology_name, service_action)
        else:
            raise ParseFailure()

    def _parse_consultDB_plan_item(self, string):
        m = re.search('^consultDB\((.+)\)$', string)
        if m:
            question_string = m.group(1)
            question = self._parse_question(question_string)
            return ConsultDBPlanItem(question)
        else:
            raise ParseFailure()

    def _parse_if_then_else_plan_item(self, string):
        m = re.search('^if (.+) then (.*) else (.*)$', string)
        if m:
            (condition_string, consequent_string, alternative_string) = m.groups()
            condition = self._parse_proposition(condition_string)
            if consequent_string != "":
                consequent = self.parse(consequent_string)
            else:
                consequent = None
            if alternative_string != "":
                alternative = self.parse(alternative_string)
            else:
                alternative = None
            return IfThenElse(condition, consequent, alternative)
        else:
            raise ParseFailure()

    def _parse_jumpto_plan_item(self, string):
        m = re.search('^jumpto\((.+)\)$', string)
        if m:
            goal_string = m.group(1)
            goal = self._parse_goal(goal_string)
            return JumpToPlanItem(goal)
        else:
            raise ParseFailure()

    def _parse_goal_proposition(self, string):
        m = re.search('^(~?)goal\((.+)\)$', string)
        if m:
            (polarity_str, goal_string) = (m.group(1), m.group(2))
            polarity = self._parse_polarity(polarity_str)
            goal = self._parse_goal(goal_string)
            return GoalProposition(goal, polarity)
        raise ParseFailure()

    def _parse_rejected_proposition(self, string):
        m = re.search('^rejected\(([^,]+)(, *(.+))?\)$', string)
        if m:
            (content_string, dummy, reason_string) = m.groups()
            rejected = self.parse(content_string)
            return RejectedPropositions(rejected, reason=reason_string)
        raise ParseFailure()

    def _parse_deprecated_service_action_terminated_proposition(self, string):
        m = re.search('^service_action_terminated\((.+)\)$', string)
        if m:
            service_action = m.group(1)
            return ServiceActionTerminatedProposition(self.ontology_name, service_action)
        raise ParseFailure()

    def _parse_deprecated_service_action_started_proposition(self, string):
        m = re.search('^service_action_started\((.+)\)$', string)
        if m:
            service_action = m.group(1)
            return ServiceActionStartedProposition(self.ontology_name, service_action)
        raise ParseFailure()

    def _parse_predicate_proposition(self, string):
        m = re.search('^(~?)(\w+)(\((.*)\))$', string)
        if m:
            (polarity_str, predicate_name, individual_string) = (m.group(1), m.group(2), m.group(4))

            if self.ontology.has_predicate(predicate_name):
                predicate = self.ontology.get_predicate(predicate_name)
                if individual_string is None or individual_string is "":
                    if not predicate.getSort().is_boolean_sort():
                        raise ParseFailure()
                    individual = None
                else:
                    individual = self._parse_individual(individual_string)
                polarity = self._parse_polarity(polarity_str)
                return PredicateProposition(predicate, individual, polarity)
        raise ParseFailure()

    def _parse_negative_individual(self, string):
        m = re.search('^~(\w+)$', string)
        if m:
            individual_value = m.group(1)
            return self.ontology.create_negative_individual(individual_value)
        raise ParseFailure()

    def _parse_individual(self, string):
        try:
            return self._parse_real_individual(string)
        except ParseFailure:
            pass
        try:
            return self._parse_integer_individual(string)
        except ParseFailure:
            pass
        try:
            return self._parse_string_individual(string)
        except ParseFailure:
            pass
        try:
            return self._parse_individual_of_enumerated_sort(string)
        except ParseFailure:
            pass
        raise ParseFailure()

    def _parse_individual_of_enumerated_sort(self, string):
        m = re.search('^(.+)$', string)
        if m:
            individual_value = m.group(1)
            try:
                sort = self.ontology.individual_sort(individual_value)
            except OntologyError:
                raise ParseFailure()
            if not sort.is_string_sort() and not sort.is_real_sort():
                return self.ontology.create_individual(individual_value)
        raise ParseFailure()

    def _parse_real_individual(self, string):
        m = re.search('^[0-9]*\.[0-9]+$', string)
        if m:
            individual_value = float(string)
            return self.ontology.create_individual(individual_value)
        else:
            raise ParseFailure()

    def _parse_integer_individual(self, string):
        m = re.search('^[0-9]+$', string)
        if m:
            individual_value = int(string)
            return self.ontology.create_individual(individual_value)
        else:
            raise ParseFailure()

    def _parse_string_individual(self, string):
        string_content = self._parse_string(string)
        individual = self.ontology.create_individual(string_content)
        return individual

    def _parse_understanding_proposition(self, string):
        m = re.search('^(\~?)und\((USR|SYS|MODEL|None),[ ]*(.+)\)$', string)
        if m:
            negation = m.group(1)
            if negation == "":
                polarity = Polarity.POS
            else:
                polarity = Polarity.NEG
            speaker = self._parse_speaker(m.group(2))
            proposition_str = m.group(3)
            proposition = self.parse(proposition_str)
            und = UnderstandingProposition(speaker, proposition, polarity)
            return und
        else:
            raise ParseFailure()

    def _parse_speaker(self, string):
        if string == Speaker.USR:
            return Speaker.USR
        elif string == Speaker.SYS:
            return Speaker.SYS
        elif string == Speaker.MODEL:
            return Speaker.MODEL
        elif string == "None":
            return None

    def _parse_yes_no_question(self, string):
        m = re.search('^\?(\w+\(\w+\))$', string)
        if m:
            proposition_string = m.group(1)
            proposition = self._parse_predicate_proposition(proposition_string)
            return YesNoQuestion(proposition)
        else:
            raise ParseFailure()

    def _parse_yes_or_no(self, string):
        m = re.search('^(yes|no)$', string)
        if m:
            yes_no_string = m.group(1)
            if yes_no_string == "yes":
                return Yes()
            return No()
        else:
            raise ParseFailure()

    def _parse_action(self, string):
        m = re.search('^(\w+)$', string)
        try:
            if m:
                action_value = m.group(1)
                return self.ontology.create_action(action_value)
        except OntologyError:
            pass
        raise ParseFailure()

    def _parse_predicate(self, string):
        m = re.search('^(\w+)$', string)
        if m:
            predicate_value = m.group(1)
            try:
                return self.ontology.get_predicate(predicate_value)
            except OntologyError:
                pass
        raise ParseFailure()

    def _parse_prop_set(self, string):
        m = re.search('^(~?)set\((.+)\)$', string)
        if m:
            (polarity_str, alts_as_string) = m.groups()
            propositions = self._parse_proposition_list(alts_as_string)
            polarity = self._parse_polarity(polarity_str)
            return PropositionSet(propositions, polarity)
        raise ParseFailure()

    def _parse_proposition_list(self, string):
        propositions = []
        string = self._strip_brackets(string)
        while self._more_data_to_read(string):
            (proposition, rest) = self._get_next_proposition(string)
            propositions.append(proposition)
            string = rest
        return propositions

    def _get_next_proposition(self, string):
        m = re.search('^(?P<prop_group>([^,]+)) *\,? *(?P<rest>.*)', string)
        if m:
            proposition = self._parse_proposition(m.group('prop_group'))
            rest = m.group('rest')
            return (proposition, rest)
        else:
            raise ParseFailure()

    def _get_next_question_param(self, string):
        m = re.search('^(?P<key>[^=]+)=(?P<value>[^=]+) *\, *(?P<rest>[^=]+=.*)$', string)
        if m:
            rest = m.group('rest')
        else:
            m = re.search('^(?P<key>[^=]+)=(?P<value>.+)$', string)
            rest = ""
        if m:
            key = m.group('key')
            value_as_string = m.group('value')
            value = self.parse_parameter(key, value_as_string)
            return (key, value, rest)
        else:
            raise ParseFailure()

    def parse_parameter(self, key, string):
        try:
            if key == "graphical_type":
                return self._parse_findout_type(string)
            elif key == "source":
                return self._parse_findout_source(string)
            elif key == "incremental":
                return self._parse_boolean(string)
            elif key == "alts":
                return self._parse_prop_set(string)
            elif key == "service_query":
                return self._parse_question(string)
            elif key == "device":
                return string
            elif key == "verbalize":
                return self._parse_boolean(string)
            elif key == "default":
                return self._parse_question(string)
            elif key == "format":
                return string
            elif key == "label_questions":
                return self._parse_list_of_questions(string)
            elif key == "sort_order":
                return self._parse_findout_sort_order(string)
            elif key == "background":
                return self._parse_predicate_list(string)
            elif key == "ask_features":
                return self._parse_predicate_list(string)
            elif key == "related_information":
                return self._parse_list_of_questions(string)
            elif key == "allow_goal_accommodation":
                return self._parse_boolean(string)
            elif key == "max_spoken_alts":
                return self._parse_integer(string)
            else:
                raise ParseError("unsupported question parameter '%s'" % key)
        except ParseFailure:
            raise ParseError("failed to parse parameter %s=%s" % (key, string))

    def _parse_predicate_list(self, string):
        string_without_brackets = self._strip_brackets(string)
        predicate_strings = [str.strip() for str in string_without_brackets.split(",")]
        return [self._parse_predicate(predicate_string) for predicate_string in predicate_strings]

    def _parse_findout_type(self, string):
        if string in [QuestionRaisingPlanItem.GRAPHICAL_TYPE_LIST, QuestionRaisingPlanItem.GRAPHICAL_TYPE_TEXT]:
            return string
        else:
            raise ParseFailure()

    def _parse_findout_source(self, string):
        if string in [QuestionRaisingPlanItem.SOURCE_SERVICE, QuestionRaisingPlanItem.SOURCE_DOMAIN]:
            return string
        else:
            raise ParseFailure()

    def _parse_findout_sort_order(self, string):
        if string in [QuestionRaisingPlanItem.ALPHABETIC]:
            return string
        else:
            raise ParseFailure()

    def _parse_boolean(self, string):
        if string.lower() == "true":
            return True
        elif string.lower() == "false":
            return False
        else:
            raise ParseFailure()

    def _parse_integer(self, string):
        m = re.search('^[0-9]+$', string)
        if m:
            individual_value = int(string)
            return individual_value
        else:
            raise ParseFailure()

    def _parse_invoke_service_action_params(self, string):
        params = {}
        string = self._strip_curly_brackets(string)
        while self._more_data_to_read(string):
            (key, value, rest) = self._get_next_invoke_service_action_param(string)
            params[key] = value
            string = rest
        return params

    def _get_next_invoke_service_action_param(self, string):
        m = re.search('^(?P<key>[^=]+)=(?P<value>[^, ]+) *\,? *(?P<rest>.*)', string)
        if m:
            key = m.group('key')
            value_as_string = m.group('value')
            if key == "postconfirm":
                value = self._parse_boolean(value_as_string)
            elif key == "preconfirm":
                value = self.parse_preconfirm_value(value_as_string)
            elif key == "downdate_plan":
                value = self._parse_boolean(value_as_string)
            else:
                raise ParseFailure()
            rest = m.group('rest')
            return (key, value, rest)
        else:
            raise ParseFailure()

    def parse_preconfirm_value(self, string):
        if string == "interrogative":
            return InvokeServiceActionPlanItem.INTERROGATIVE
        elif string == "assertive":
            return InvokeServiceActionPlanItem.ASSERTIVE
        else:
            raise ParseFailure()

    def _strip_brackets(self, bracketed_string):
        matcher = re.search('^\[(.*)\]$', bracketed_string)
        if matcher:
            return matcher.group(1)
        else:
            raise ParseError("_strip_brackets failed for '%s'" % bracketed_string)

    def _strip_curly_brackets(self, bracketed_string):
        matcher = re.search('^\{(.*)\}$', bracketed_string)
        if matcher:
            return matcher.group(1)

    def _more_data_to_read(self, string):
        return string != ""

    def _parse_polarity(self, string):
        if string == "":
            return Polarity.POS
        elif string == "~":
            return Polarity.NEG
        else:
            raise Exception("expected polarity string but got '%s'" % string)



class RequiredEntity(object):
    def __init__(self, name):
        self._name = name

    @property
    def name(self):
        return self._name

    @property
    def is_propositional(self):
        return False

    @property
    def is_sortal(self):
        return False

    def __repr__(self):
        return "%s(%r)" % (self.__class__.__name__, self._name)

    def __eq__(self, other):
        return bool(isinstance(other, self.__class__) and self._name == other.name)


class RequiredSortalEntity(RequiredEntity):
    @property
    def is_sortal(self):
        return True


class RequiredPropositionalEntity(RequiredEntity):
    @property
    def is_propositional(self):
        return True



import argparse
import re

from tala.config import OverriddenDddConfig, BackendConfig


def add_common_backend_arguments(parser):
    def parse_ddd_config(string):
        match = re.search('^(.+):(.+)$', string)
        if match:
            ddd_name, path = match.group(1), match.group(2)
            return OverriddenDddConfig(ddd_name, path)
        else:
            raise argparse.ArgumentTypeError("Expected DDD configs on the format 'DDD:CONFIG' but got '%s'." % string)

    parser.add_argument(
        "--config",
        dest="config",
        default=None,
        help="override the default backend config %r" % BackendConfig.default_name()
    )
    parser.add_argument(
        "--ddd-config",
        dest="overridden_ddd_config_paths",
        type=parse_ddd_config,
        nargs="+",
        help="override a DDD config",
        metavar="DDD:CONFIG"
    )



from mock import patch

from tala.config import BackendConfig
from tala.ddd.loading import ddd_loader
from tala.ddd.loading.ddd_loader import DDDLoader
from tala.testing.ddd_mocker import DddMockingTestCase


class TestDDDLoader(DddMockingTestCase):
    def setUp(self):
        self._backend_config = BackendConfig.default_config()
        DddMockingTestCase.setUp(self)

    def tearDown(self):
        DddMockingTestCase.tearDown(self)

    def _when_load_is_called(self, ddd_name, languages=None):
        with patch('%s.warnings' % ddd_loader.__name__) as mock_warnings:
            self._mock_warnings = mock_warnings
            languages = languages or ["eng"]
            mock_ddd_loader = DDDLoader(ddd_name, self._mock_ddd_config, languages)
            self._result = mock_ddd_loader.load()

    def test_loading_with_rasa_posts_warning_message(self):
        self._given_ontology_py_file("mockup_app/ontology.py")
        self._given_domain_py_file("mockup_app/domain.py")
        self._given_service_interface_xml_file("mockup_app/service_interface.xml")
        self._given_device_py_file("mockup_app/device.py")
        self._given_mocked_ddd_config(enable_rasa_nlu=True)
        self._when_load_is_called("mockup_app")
        self._then_warning_message_is_posted(
            "The support for RASA NLU is still in BETA. Talk gently. "
            "It is currently enabled in DDD 'mockup_app'."
        )

    def _then_warning_message_is_posted(self, expected_warning):
        self._mock_warnings.warn.assert_any_call(expected_warning)



ENGLISH = "eng"
RASA_ENGLISH = "en"
SWEDISH = "sv"
RASA_SWEDISH = "sv"
ITALIAN = "it"
FRENCH = "fre"
DUTCH = "dut"
CHINESE = "chi"
SPANISH = "spa"
RASA_SPANISH = "es"

SUPPORTED_RASA_LANGUAGES = [
    ENGLISH,
    SWEDISH,
    SPANISH
]

PREPACKAGED_SPACY_LANGUAGES = [
    RASA_ENGLISH,
    RASA_SPANISH
]

RASA_LANGUAGE = {
    ENGLISH: RASA_ENGLISH,
    SWEDISH: RASA_SWEDISH,
    SPANISH: RASA_SPANISH
}

SUPPORTED_LANGUAGES = [ENGLISH, SWEDISH, ITALIAN, FRENCH, DUTCH, CHINESE, SPANISH]



#!/usr/bin/env python2.7

from setuptools import find_packages, setup

try:
    from setuptools.command.build_py import build_py
except ImportError:
    from distutils.command.build_py import build_py  # noqa: F401


def readme():
    with open("README.md") as f:
        return f.read()


if __name__ == "__main__":
    setup(
        use_scm_version={"write_to": "tala/installed_version.py"},
        setup_requires=["setuptools_scm"],
        name="tala",
        description="Design dialogue domain descriptions (DDDs) for TDM",
        long_description=readme(),
        long_description_content_type="text/markdown",
        classifiers=[
            "Operating System :: OS Independent",
            "Programming Language :: Python :: 2.7",
        ],
        keywords="tala tdm ddd ddds dialogue conversation AI",
        packages=find_packages(exclude=["tala/ddds", "test", "*.test", "test.*", "*.test.*"]),
        package_dir={"tala": "tala"},
        package_data={
            "tala": [
                "ddd/maker/templates/*.txt",
                "ddd/maker/templates/*.xml",
                "ddd/schemas/grammar.xsd",
                "ddd/schemas/grammar_rgl.xsd",
                "ddd/schemas/ontology.xsd",
                "ddd/schemas/service_interface.xsd",
            ]
        },
        scripts=[],
        entry_points={
            "console_scripts": [
                "tala = tala.cli.console_script:main",
            ],
        },
        url="http://www.talkamatic.se",
        author="Talkamatic",
        author_email="dev@talkamatic.se",
        install_requires=[
            "Jinja2~=2.10",
            "pathlib==1.0.1",
            "lxml==4.2.5",
            "iso8601==0.1.12",
            "python-magic==0.4.15",
        ],
        dependency_links=[],
    )



from tala.model.device import ParameterField
from tala.utils.as_json import JSONLoggable


class UnexpectedParameterFieldException(Exception):
    pass


class UnexpectedActionException(Exception):
    pass


class UnexpectedQueryException(Exception):
    pass


class UnexpectedValidatorException(Exception):
    pass


class DuplicateNameException(Exception):
    pass


class FailureReasonsNotAllowedException(Exception):
    pass


class UnsupportedServiceInterfaceTarget(Exception):
    pass


class ServiceInterface(JSONLoggable):
    def __init__(self, actions, queries, entity_recognizers, validators):
        self._validate(actions)
        self._actions = {action.name: action for action in actions}
        self._validate(queries)
        self._queries = {query.name: query for query in queries}
        self._validate(entity_recognizers)
        self._entity_recognizers = {recognizer.name: recognizer for recognizer in entity_recognizers}
        self._validate(validators)
        self._validators = {validator.name: validator for validator in validators}

    def _validate(self, specific_interfaces):
        names = [interface.name for interface in specific_interfaces]
        if not self._all_unique(names):
            raise DuplicateNameException(
                "Expected all names to be unique among %s but they weren't" % specific_interfaces
            )

    def _all_unique(self, all_names):
        unique_names = set(all_names)
        return len(all_names) == len(unique_names)

    @property
    def actions(self):
        return self._actions.values()

    def get_action(self, name):
        if not self.has_action(name):
            raise UnexpectedActionException(
                "Expected one of the known actions %s but got '%s'" % (self._actions.keys(), name)
            )
        return self._actions[name]

    def has_action(self, name):
        return name in self._actions

    def get_query(self, name):
        if not self.has_query(name):
            raise UnexpectedQueryException(
                "Expected one of the known queries %s but got '%s'" % (self._queries.keys(), name)
            )
        return self._queries[name]

    def has_query(self, name):
        return name in self._queries

    def get_validator(self, name):
        if not self.has_validator(name):
            raise UnexpectedValidatorException(
                "Expected one of the known validators %s but got '%s'" % (self._validators.keys(), name)
            )
        return self._validators[name]

    def has_validator(self, name):
        return name in self._validators

    @property
    def queries(self):
        return self._queries.values()

    @property
    def validators(self):
        return self._validators.values()

    @property
    def entity_recognizers(self):
        return self._entity_recognizers.values()

    def __repr__(self):
        return "%s(actions=%s, queries=%s, validators=%s, entity_recognizers=%s)" % (
            self.__class__.__name__, self.actions, self.queries, self.validators, self.entity_recognizers
        )

    def __eq__(self, other):
        def has_all(these, those):
            return all(this in those for this in these) and all(that in these for that in those)

        return bool(
            isinstance(other, self.__class__) and has_all(self.actions, other.actions)
            and has_all(self.queries, other.queries) and has_all(self.validators, other.validators)
            and has_all(self.entity_recognizers, other.entity_recognizers)
        )


class SpecificServiceInterface(JSONLoggable):
    def __init__(self, interface_type, name, target):
        super(SpecificServiceInterface, self).__init__()
        self._interface_type = interface_type
        self._name = name
        self._target = target

    @property
    def interface_type(self):
        return self._interface_type

    @property
    def name(self):
        return self._name

    @property
    def target(self):
        return self._target

    def __repr__(self):
        return "%s(%r, %r, %r)" % (self.__class__.__name__, self.interface_type, self.name, self.target)

    def __eq__(self, other):
        return bool(
            isinstance(other, self.__class__) and self.interface_type == other.interface_type
            and self.name == other.name and self.target == other.target
        )

    def ensure_target_is_not_frontend(self):
        if self.target.is_frontend:
            raise UnsupportedServiceInterfaceTarget(
                "Expected a non-frontend target for service interface '%s' but got a frontend target." % self.name
            )


class ParameterizedSpecificServiceInterface(SpecificServiceInterface):
    def __init__(self, interface_type, name, target, parameters):
        super(ParameterizedSpecificServiceInterface, self).__init__(interface_type, name, target)
        self._parameters = parameters

    @property
    def parameters(self):
        return self._parameters

    def __repr__(self):
        return "%s(%r, %r, parameters=%s)" % (self.__class__.__name__, self.name, self.target, self.parameters)

    def __eq__(self, other):
        return bool(
            isinstance(other, self.__class__) and self.name == other.name and self.target == other.target
            and self.parameters == other.parameters
        )


class BaseActionInterface(ParameterizedSpecificServiceInterface):
    def __init__(self, name, target, parameters, failure_reasons):
        super(BaseActionInterface, self).__init__("action", name, target, parameters)
        self._failure_reasons = failure_reasons
        self._validate_target_and_failure_reasons()

    def _validate_target_and_failure_reasons(self):
        if self.target.is_frontend:
            if self.failure_reasons:
                failure_reason_names = [reason.name for reason in self.failure_reasons]
                raise FailureReasonsNotAllowedException(
                    "Expected no failure reasons for action '%s' with target 'frontend', but got %s" %
                    (self.name, failure_reason_names)
                )

    @property
    def failure_reasons(self):
        return self._failure_reasons

    @property
    def is_play_audio_action(self):
        raise NotImplementedError("Needs to be implemented")

    def __repr__(self):
        return "%s(%r, %r, parameters=%s, failure_reasons=%s)" % (
            self.__class__.__name__, self.name, self.target, self.parameters, self.failure_reasons
        )

    def __eq__(self, other):
        return bool(
            isinstance(other, self.__class__) and self.name == other.name and self.target == other.target
            and self.parameters == other.parameters and self.failure_reasons == other.failure_reasons
        )


class ServiceActionInterface(BaseActionInterface):
    @property
    def is_play_audio_action(self):
        return False


class PlayAudioActionInterface(BaseActionInterface):
    def __init__(self, name, target, parameters, audio_url_parameter):
        parameters = parameters + [audio_url_parameter]
        super(PlayAudioActionInterface, self).__init__(name, target, parameters, failure_reasons=[])
        self._audio_url_parameter = audio_url_parameter

    @property
    def audio_url_parameter(self):
        return self._audio_url_parameter

    @property
    def is_play_audio_action(self):
        return True

    def __repr__(self):
        return "%s(%r, %r, parameters=%s, audio_url_parameter=%s)" % (
            self.__class__.__name__, self.name, self.target, self.parameters, self.audio_url_parameter
        )

    def __eq__(self, other):
        return bool(
            isinstance(other, self.__class__) and self.name == other.name and self.target == other.target
            and self.parameters == other.parameters and self.audio_url_parameter == other.audio_url_parameter
        )


class ServiceQueryInterface(ParameterizedSpecificServiceInterface):
    def __init__(self, *args, **kwargs):
        super(ServiceQueryInterface, self).__init__("query", *args, **kwargs)
        self.ensure_target_is_not_frontend()


class ServiceValidatorInterface(ParameterizedSpecificServiceInterface):
    def __init__(self, *args, **kwargs):
        super(ServiceValidatorInterface, self).__init__("validator", *args, **kwargs)
        self.ensure_target_is_not_frontend()


class ServiceEntityRecognizerInterface(SpecificServiceInterface):
    def __init__(self, *args, **kwargs):
        super(ServiceEntityRecognizerInterface, self).__init__("entity_recognizer", *args, **kwargs)
        self.ensure_target_is_not_frontend()

    def __repr__(self):
        return "%s(%r, %r)" % (self.__class__.__name__, self.name, self.target)


class ServiceImplicationInterface(SpecificServiceInterface):
    def __init__(self, *args, **kwargs):
        super(ServiceImplicationInterface, self).__init__("implication", *args, **kwargs)

    def __repr__(self):
        return "%s(%r, %r)" % (self.__class__.__name__, self.name, self.target)


class AbstractServiceParameter(JSONLoggable):
    def __init__(self, name, format):
        self._name = name
        self._format = format

    @property
    def name(self):
        return self._name

    @property
    def format(self):
        return self._format

    @property
    def is_optional(self):
        raise NotImplementedError("Needs to be implemented")


class ServiceParameter(AbstractServiceParameter):
    VALID_FORMATS = [ParameterField.VALUE, ParameterField.GRAMMAR_ENTRY]

    def __init__(self, name, format=None, is_optional=None):
        is_optional = is_optional or False
        format = format or ParameterField.VALUE
        if format not in self.VALID_FORMATS:
            raise UnexpectedParameterFieldException(
                "Expected format as one of %s but got '%s' for parameter '%s'" % (self.VALID_FORMATS, format, name)
            )
        super(ServiceParameter, self).__init__(name, format)
        self._is_optional = is_optional

    @property
    def is_optional(self):
        return self._is_optional

    def __repr__(self):
        return "%s(%r, format=%r, is_optional=%r)" % (self.__class__.__name__, self.name, self.format, self.is_optional)

    def __eq__(self, other):
        return bool(
            isinstance(other, self.__class__) and self.name == other.name and self.format == other.format
            and self.is_optional == other.is_optional
        )


class AudioURLServiceParameter(AbstractServiceParameter):
    def __init__(self, name):
        format = ParameterField.GRAMMAR_ENTRY
        super(AudioURLServiceParameter, self).__init__(name, format)

    @property
    def is_optional(self):
        return False

    def __repr__(self):
        return "%s(%r, format=%r)" % (self.__class__.__name__, self.name, self.format)

    def __eq__(self, other):
        return bool(isinstance(other, self.__class__) and self.name == other.name and self.format == other.format)


class ActionFailureReason(JSONLoggable):
    def __init__(self, name):
        super(ActionFailureReason, self).__init__()
        self._name = name

    @property
    def name(self):
        return self._name

    def __repr__(self):
        return "%s(%r)" % (self.__class__.__name__, self.name)

    def __eq__(self, other):
        return bool(isinstance(other, self.__class__) and self.name == other.name)


class ServiceTarget(JSONLoggable):
    @property
    def is_device_module(self):
        return False

    @property
    def is_frontend(self):
        return False

    @property
    def is_http(self):
        return False

    def __repr__(self):
        return "%s()" % self.__class__.__name__

    def __eq__(self, other):
        return isinstance(other, self.__class__)

    def __ne__(self, other):
        return not (self == other)


class DeviceModuleTarget(ServiceTarget):
    def __init__(self, device):
        super(DeviceModuleTarget, self).__init__()
        self._device = device

    @property
    def device(self):
        return self._device

    @property
    def is_device_module(self):
        return True

    def __repr__(self):
        return "%s(%r)" % (self.__class__.__name__, self.device)

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.device == other.device


class FrontendTarget(ServiceTarget):
    @property
    def is_frontend(self):
        return True


class HttpTarget(ServiceTarget):
    def __init__(self, endpoint):
        super(HttpTarget, self).__init__()
        self._endpoint = endpoint

    @property
    def endpoint(self):
        return self._endpoint

    @property
    def is_http(self):
        return True

    def __repr__(self):
        return "%s(%r)" % (self.__class__.__name__, self.endpoint)

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.endpoint == other.endpoint



def abstract_gf_filename(ddd_name):
    return "%s.gf" % ddd_name


def natural_language_gf_filename(ddd_name, language_code):
    return "%s_%s.gf" % (ddd_name, language_code)


def semantic_gf_filename(ddd_name):
    return "%s_sem.gf" % ddd_name
